package Hawat;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

#$ENV{SSL_CLIENT_S_DN}    ="/C=CZ/O=CESNET/CN=Jan Mach/unstructuredName=5832";
#$ENV{SSL_CLIENT_S_DN_CN} ="Jan Mach";
#$ENV{SSL_CLIENT_S_DN_O}  ="CESNET";

use Data::Dumper;
use HTML::Entities;
use Cwd qw(abs_path);
use File::Basename;
use Storable qw(dclone);
use JSON;
use Number::Format qw(:subs :vars);
use Value::Convertor;
use Catalyst::Runtime 5.80;
use Mentat::Core qw(:c_hawat);
use Color::Palette;

my $PKG_VERSION_FILE;

# Load additional library locations, unless this is a package installation
unless (__FILE__ =~ m|^/var/mentat/www|) {
    # Tarball installation
    if (__FILE__ =~ m|^/opt/mentat/bin|) {
        use lib (abs_path("/opt/libcesnet-toolkit-perl/lib") =~ m{^(/.*)$})[0];
        use lib (abs_path("/opt/mentat/lib") =~ m{^(/.*)$})[0];
        $PKG_VERSION_FILE = '/opt/mentat/doc/hawat/version';
    }
    # Development environment
    else {
        use lib (abs_path(dirname(__FILE__).'/../../../lib') =~ m{^(/.*)$})[0];
    }
}
else {
    $PKG_VERSION_FILE = '/usr/share/doc/mentat/hawat/version';
}

# Set flags and add plugins for the application
#         -Debug: activates the debug mode for very useful log messages
#   ConfigLoader: will load the configuration from a Config::General file in the
#                 application's home directory
# Static::Simple: will serve static files from the application's root directory

use Catalyst qw/
    -Debug
    ConfigLoader
    Static::Simple

    Email

    Session
    Session::Store::FastMmap
    Session::State::Cookie

    Authentication
    Authorization::Roles
    Authorization::ACL
/;

extends 'Catalyst';

# Configure the application.
#
# Note that settings in hawat.{conf|json} (or other external
# configuration file that you set up manually) take precedence
# over this when using ConfigLoader. Thus configuration
# details given here can function as a default configuration,
# with an external configuration file acting as an override for
# local deployment.

__PACKAGE__->config(
        email => ['Sendmail'],
        #some_value => 'Hawat',
        #'Hawat::Controller::Report' => { some_value => 'Hawat' },
    );
has 'pkg_version' => (is => 'ro', isa => 'Str', writer => '_set_pkgv');

# Start the application
__PACKAGE__->setup();

# Configure code and package versions
our $VERSION = '0.1.1';

# Define meaningful default configuration values
my $DEFAULTS = {
        devel => 0,
        log_threshold => 'INFO',
    };

# Load global configurations from config file
my $CFG_G = Hawat->config;

# Setup configurations for use in this module
our $DEVEL           = Hawat->cfg('devel');
my  $LOG_THRESHOLD   = Hawat->cfg('log_threshold');
my  $AUTHENTICATION  = Hawat->cfg('authentication');
my  $AUTHORIZATION   = Hawat->cfg('authorization');
my  $ACL             = Hawat->cfg('acl');
my  $ACL_VALIDATOR   = {};
my  %VALID_CONSTANTS = (
        'HAWAT_FORM_PREVIEW'      => 1,
        'HAWAT_FORM_AUTOFILL'     => 1,
        'HAWAT_FORM_CANCEL'       => 1,
        'HAWAT_ACCOUNT_ANONYMOUS' => 1,
    );

# Setup the loger
Log::Loger->filter('filter1','threshold', $LOG_THRESHOLD, 'r');
Log::Loger->writer('writer1', 'file', Hawat->cfg('log_file'), undef, 'r');
Log::Loger->channel('filter1','writer1','accept');

if ($DEVEL) {
    use Data::Dumper;
}

#-------------------------------------------------------------------------------
# ACL configuration
#-------------------------------------------------------------------------------

# Authorization may be turned off via configuration file
if ($AUTHORIZATION) {
    # Add rules that permit access to the root controller internal auto, begin and end unconditionally
    __PACKAGE__->acl_allow_root_internals;
    # Add custom ACL rules based on paths
    while (my ($path, $roles) = each(%{$ACL->{acl_url}})) {
        # Configure the Catalysts native ACL system
        __PACKAGE__->deny_access_unless_any($path, $roles);
        # Configure custom ACL validator
        $ACL_VALIDATOR->{$path} = [['check_roles', @$roles]];
    }
    while (my ($path, $rules) = each(%{$ACL->{acl_custom}})) {
        my $r = [];
        if ($rules->{role}) {
            push(@$r, ['check_roles', @{$rules->{role}}]);
        }
        if ($rules->{group}) {
            foreach my $gm (@{$rules->{group}}) {
                push(@$r, ['check_management']) if lc($gm) eq 'manager';
                push(@$r, ['check_membership']) if lc($gm) eq 'member';
            }
        }
        if ($rules->{user}) {
            foreach my $um (@{$rules->{user}}) {
                push(@$r, ['check_me']) if lc($um) eq 'me';
            }
        }
        $ACL_VALIDATOR->{$path} = $r;
    }
    #Log::Loger->info(__PACKAGE__, "ACL VALIDATOR: " . Dumper($ACL_VALIDATOR));
}

#-------------------------------------------------------------------------------

# current_time()
#
#  Usage     : [% c.current_time() %]   # Use from template
#  Purpose   : Return current timestamp
#  Arguments : None
#  Returns   : INTEGER
#  Throws    : Nothing

sub current_time()
{
    my $c = shift;
    return time;
}

# time_before($)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    :

sub time_before($;$)
{
    my $c = shift;
    my ($period, $mod) = @_;

    my %periods = (
        '1week'  => (60*60*24*7),
        '2weeks' => (60*60*24*7*2),
        '4weeks' => (60*60*24*7*4),
    );

    my $t = time;
    if ($period and $periods{$period}) {
        $t -= $periods{$period};
    }
    if ($mod) {
        $t = $t - ($t % (60*60*24));
    }
    return Value::Convertor->unixs_to_ldatestr($t);

}

# const($;$$)
#
#  Usage     : [% c.const('CONSTANT_NAME') %]   # Use from template
#  Purpose   : Return global constant, intended to be used from view templates
#  Arguments : STRING $key - name of the configuration value [MANDATORY]
#  Returns   : MIXED $constant
#  Throws    : Nothing

sub const($)
{
    my $c = shift;
    my ($const,) = @_;
    die "Requested invalid constant '$const'\n" unless exists $VALID_CONSTANTS{$const};
    no strict 'refs';
    return &{$const}();
}

# cfg($;$$)
#
#  Usage     : my $config = Hawat->cfg('key', 'section', $default); # OR
#              my $config = $c->cfg('key', 'section', $default);
#  Purpose   : Return single configuration value
#  Arguments : STRING $key - name of the configuration value [MANDATORY]
#              STRING $section - name of the section, in which to look for more specific value [OPTIONAL]
#              HASH REFERENCE $default - component specific default value [OPTIONAL]
#  Returns   : MIXED $config
#  Throws    : Nothing

sub cfg($;$$)
{
    my $c = shift;
    my ($key, $section, $default) = @_;

    $CFG_G = Hawat->config;

    my $result = $DEFAULTS->{$key};
    $result = $default if defined $default;
    $result = $CFG_G->{$key} if exists $CFG_G->{$key};
    $result = $CFG_G->{$section}->{$key} if $section and exists $CFG_G->{$section}->{$key};
    return $result;
}

# get_current_user_id()
#
#  Usage     : my $id = $c->get_current_user_id();
#  Purpose   : Get the ID of user currently logged in to application
#  Arguments : None
#  Returns   : STRING $id if the user is logged in, STRING 'Anonymous' otherwise
#  Throws    : Nothing

sub get_current_user_id()
{
    my ($c) = @_;

    # User authentication may be turned off via configuration file
    return $c->user()->{HAWAT_ATTR_MID()} if $AUTHENTICATION;
    return HAWAT_ACCOUNT_ANONYMOUS;
}

# user_is_activated()
#
#  Usage     : my $rv = $c->user_is_activated();
#  Purpose   : Check, if the user account was activated
#  Arguments : None
#  Returns   : STRING $id if the user is logged in, STRING 'Anonymous' otherwise
#  Throws    : Nothing

sub user_is_activated()
{
    my ($c) = @_;

    return 0 unless $c->check_user_roles($c->cfg('acl')->{base_role});
    return 1;
}

# dump_variable($)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub dump_variable($)
{
    my ($c, $variable) = @_;

    $Data::Dumper::Indent = 2;
    my $dump = Dumper($variable);
    #$c->log->debug('DUMP VARIABLE:'.$dump);
    $dump =~ s|^\$VAR1 = ||;
    $dump = encode_entities($dump);
    $dump =~ s|\n\s*\n|\n|g;
    $dump =~ s|\n\n|\n|g;
    $dump =~ s|\n|<br>\n|g;
    $dump =~ s| |&nbsp;|g;
    return $dump;
}

# tj()
#
#  Usage     : my $json = Hawat->tj();
#              my $json = $c->tj();
#  Purpose   :
#  Arguments : NONE
#  Returns   : STRING $json
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

sub tj()
{
    my ($c, $variable) = @_;
    return $variable if not $variable or not ref $variable;
    return to_json($variable, {utf8 => 1, pretty => 1});
}

# nl2br()
#
#  Usage     : my $text = Hawat->nl2br($text);
#              my $text = $c->nl2br($text);
#  Purpose   :
#  Arguments : NONE
#  Returns   : STRING $text
#  Throws    : Nothing

sub nl2br()
{
    my ($c, $text) = @_;
    $text =~ s/\r?\n/<br>/g;
    return $text;
}

# seq()
#
#  Usage     : my $copy = Hawat->seq();
#              my $copy = $c->seq();
#  Purpose   :
#  Arguments : NONE
#  Returns   : INTEGER $sequence
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

sub seq()
{
    my $c = shift;
    return $c->session->{sequence};
}

# eo()
#
#  Usage     : my $copy = Hawat->eo();
#              my $copy = $c->eo();
#  Purpose   :
#  Arguments : NONE
#  Returns   : ENUM ['odd','even']
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

{
my @__EO = ('odd','even');
sub eo($)
{
    my $c = shift;
    my ($first,) = @_;
    @__EO = ('odd','even') if $first;
    my $rv = shift @__EO;
    push @__EO, $rv;
    return $rv;
}
}

# eor()
#
#  Usage     : my $copy = Hawat->eor($realm);
#              my $copy = $c->eor($realm);
#  Purpose   :
#  Arguments : NONE
#  Returns   : ENUM ['odd','even']
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

{
my %__EOR = ();
sub eor($)
{
    my $c = shift;
    my ($realm,) = @_;
    die "Realm is mandatory" unless $realm;
    $__EOR{$realm} = ['odd','even'] unless exists $__EOR{$realm};
    my $rv = shift @{$__EOR{$realm}};
    push @{$__EOR{$realm}}, $rv;
    return $rv;
}
}

# dc($)
#
#  Usage     : my $copy = Hawat->dc($value);
#              my $copy = $c->dc($value);
#  Purpose   : Make deep copy of given value - helper method
#  Arguments : MIXED $value
#  Returns   : MIXED $copy
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

sub dc($)
{
    my $c = shift;
    return dclone(shift);
}

# cp($@)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing
#  Comments  :

sub cp($@)
{
    my $c = shift;
    my $size = shift;
    confess "Color palete size must be given as argument" unless $size;

    my @palette = ();
    my @tmp = Color::Palette->palette($size);

    foreach my $c (@tmp) {
        my ($color, $rgb) = @$c;
        push(@palette, "#"."$rgb");
    }
    return \@palette;
}

# ic($@)
#
#  Usage     :
#  Purpose   : Make indirect call of helper method
#  Arguments :
#  Returns   :
#  Throws    : Nothing
#  Comments  :

sub ic($@)
{
    my $c = shift;
    my $method = shift;
    return $c->$method(@_) if $c->can($method);
    return $c->conv($method, @_);
}

# urlify($)
#
#  Usage     : my $result = Hawat->urlify($value); # OR
#              my $result = $c->urlify($value);
#  Purpose   :
#  Arguments : STRING $id
#  Returns   : STRING $result
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#

sub urlify($)
{
    my $c = shift;
    my $value = shift;
    $value =~ s/[^-a-zA-Z0-9]/-/g;
    return return $value;
}

# sn()
#
#  Usage     : my $json = Hawat->sn();
#              my $json = $c->sn();
#  Purpose   :
#  Arguments : NONE
#  Returns   :
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to make local copy of various variable (request params etc.)

sub sn($)
{
    my ($c, $networks) = @_;
    return $networks unless $networks;
    my @result = sort { $a->{'first'} <=> $b->{'first'} }
                 sort { $a->{'last'}  <=> $b->{'last'} } @$networks;
    return \@result;
}

# unesc($)
#
#  Usage     : my $result = Hawat->unesc($ip); # OR
#              my $result = $c->unesc($ip);
#  Purpose   :
#  Arguments : STRING $value
#  Returns   : STRING $result
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to format large integers to make them better readable,
#              for example 1200300 becomes 1.200.300

sub unesc($)
{
    my $c = shift;
    my $value = shift;
    $value =~ s/\(dot\)/./g;
    $value =~ s/\(at\)/@/g;
    return return $value;
}

# fmtn($)
#
#  Usage     : my $result = Hawat->fmtn($number); # OR
#              my $result = $c->fmtn($number);
#  Purpose   : Format number nicelly - helper method
#  Arguments : INTEGER $number
#  Returns   : STRING $result
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to format large integers to make them better readable,
#              for example 1200300 becomes 1.200.300

sub fmtn($)
{
    my $c = shift;
    return format_number(shift);
}

# fmtnc($)
#
#  Usage     : my $result = Hawat->fmtnc($number, ''); # OR
#              my $result = $c->fmtnc($number);
#  Purpose   : Format number nicelly - helper method
#  Arguments : INTEGER $number
#  Returns   : STRING $result
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to format large integers to make them better readable,
#              for example 1200300 becomes 1.200.300

sub fmtnc($$)
{
    my $c = shift;
    my ($size, $type) = @_;
    $type = 'kb' unless $type;

    my $div = 1024;
    my $sup = 'KB';
    if (lc($type) eq 'kb') {
        $div = 1024; $sup = 'KB';
    }
    if (lc($type) eq 'mb') {
        $div = 1024 * 1024; $sup = 'MB';
    }
    if (lc($type) eq 'gb') {
        $div = 1024 * 1024 * 1024; $sup = 'GB';
    }
    if (lc($type) eq 'tb') {
        $div = 1024 * 1024 * 1024 * 1024; $sup = 'TB';
    }
    return format_number(($size/$div), 2) . " $sup";
}

# pctg($)
#
#  Usage     : my $result = Hawat->pctg($part, $whole); # OR
#              my $result = $c->pctg($part, $whole);
#  Purpose   : Calculate the percentage
#  Arguments : INTEGER $part
#              INTEGER $whole
#  Returns   : STRING $result
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering
#              process to format large integers to make them better readable,
#              for example 1200300 becomes 1.200.300

sub pctg($$)
{
    my $c = shift;
    my ($part, $whole) = @_;
    return 'NaN' unless $whole;
    #return sprintf("%6.2f", (($part/$whole)*100));
    return format_number(($part/$whole)*100, 2);
}

# isin($$)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing
#  Comments  : This method is used as helper method during template rendering

sub isin($$)
{
    my $c = shift;
    my ($array, $value) = @_;
    return 0 unless $array and @$array;
    my %hash = map { $_ => 1 } @$array;
    return ($hash{$value}) ? 1 : 0;
}

# conv($@)
#
#  Usage     : my $result = Hawat->conv($conversion, @args); # OR
#              my $result = $c->conv($conversion, @args);
#  Purpose   : Convert value with given conversion method - helper method
#  Arguments : STRING $conv - name of the conversion method [MANDATORY]
#              MIXED @args - conversion arguments [MANDATORY]
#  Returns   : MIXED $result
#  Throws    : Nothing by itself
#  Comments  : This method is used as helper method during template rendering
#              process to use interface of Value::Convertor

sub conv($)
{
    my $c = shift;
    my ($conversion, @args) = @_;
    return 'NOARGS' unless @args;
    return Value::Convertor->$conversion(@args);
}

# cwrap($)
#
#  Usage     : my $result = Hawat->wrap($when, $what, $into); # OR
#              my $result = $c->wrap($when, $what, $into);
#  Purpose   : Conditionally wrap given string into given HTML markup
#  Arguments : BOOL $when - condition [MANDATORY]
#              STRING $what - what to wrap [MANDATORY]
#              STRING $into - into what to wrap [MANDATORY]
#              STRING $attrs - additional HTML element attributes, e.g. class [OPTIONAL]
#  Returns   : STRING $result
#  Throws    : Nothing

sub cwrap($$$$;$)
{
    my $c = shift;
    my ($when, $what, $into,$attrs) = @_;
    return $what unless $when;
    return "<$into".(($attrs)?" $attrs":'').">$what</$into>";
}

# acl()
#
#  Usage     : my $acl = Hawat->acl(); # OR
#              my $acl = $c->acl();
#  Purpose   : Fetch the ACL validator object
#  Arguments : None
#  Returns   : HASH
#  Throws    : Nothing

sub acl()
{
    my $c = shift;
    return $ACL_VALIDATOR;
}

# check_path_acl(;$$)
#
#  Usage     : my $result = Hawat->check_path_acl($path, $args); # OR
#              my $result = $c->check_path_acl($path, $args);    # OR from template
#              [% IF c.check_path_acl('/url/location') %][% do_something %][% END %]
#  Purpose   : Check ACL policies for given URI
#  Arguments : STRING $path - URI location [MANDATORY]
#              HASH $args - additional arguments [OPTIONAL]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#                  1 if the URI was not ACL protected
#  Throws    : Nothing

sub check_path_acl($;$)
{
    my ($c, $path, $args) = @_;
    die "Path must be given as argument" unless $path;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # Strip trailing slash from path, if necessary
    if ($path =~ m\.+/$\) { $path =~ s\/$\\; }
    # Add leading slash to path, if necessary
    $path = '/'.$path unless ($path =~ m\^/\);

    my $opath = $path;
    my $result = 1;
    while ($path) {
        # Check, if the path is controlled by ACL
        if (exists $ACL_VALIDATOR->{$path}) {
            foreach my $rule (@{$ACL_VALIDATOR->{$path}}) {
                my ($checker, @a) = @$rule;
                # Perform the role check
                if ($checker eq 'check_roles') {
                    if ($c->check_roles(@a)) {
                        $result = 1; goto CHECK_ACL_END;
                    }
                }
                # Perform the group management check
                elsif ($checker eq 'check_management') {
                    confess "Missing argument to check 'management' for path '$opath'" unless $args and ref $args eq 'HASH' and exists $args->{g};
                    if ($c->check_management($args->{g})) {
                        $result = 1; goto CHECK_ACL_END;
                    }
                }
                # Perform the group membership check
                elsif ($checker eq 'check_membership') {
                    confess "Missing argument to check 'membership' for path '$opath'" unless $args and ref $args eq 'HASH' and exists $args->{g};
                    if ($c->check_membership($args->{g})) {
                        $result = 1; goto CHECK_ACL_END;
                    }
                }
                # Perform the user check
                elsif ($checker eq 'check_me') {
                    confess "Missing argument to check 'me' for path '$opath'" unless $args and ref $args eq 'HASH' and exists $args->{u};
                    if ($c->check_me($args->{u})) {
                        $result = 1; goto CHECK_ACL_END;
                    }
                }
            }

            # Exact path is defined
            $result = 0; goto CHECK_ACL_END;
        }
        # Perhaps the restriction was defined on higher layer, so strip the last
        # chunk from path and check again
        $path =~ s\/[^/]*$\\;
    }
    CHECK_ACL_END:
    #Log::Loger->info(__PACKAGE__, "Path ACL check for '$opath' => '$path': ".(($result)?"GRANTED":"DENIED"));
    $c->log->debug("Path ACL check for '$opath' => '$path': ".(($result)?"GRANTED":"DENIED"));
    return $result;
}

# check_acl(;$$)
#
#  Usage     : my $result = Hawat->check_acl($args); # OR
#              my $result = $c->check_acl($args);    # OR from template
#              [% IF c.check_acl('/url/location') %][% do_something %][% END %]
#  Purpose   : Check ACL policies for current URI and current user roles
#  Arguments : STRING $path - URI location [OPTIONAL, defaults to current action]
#              INTEGER $default - default result, in case the URI is not ACL protected [OPTIONAL]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#                  $default if the URI was not ACL protected
#                  1 if the URI was not ACL protected and $default was not defined
#  Throws    : Nothing

sub check_acl(;$)
{
    my ($c, $args) = @_;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # By default, set the path to current action
    my $path = $c->action->private_path;

    # Perform the check
    return $c->check_path_acl($path, $args);
}

# list_roles()
#
#  Usage     : my $result = Hawat->list_roles(); # OR
#              my $result = $c->list_roles();    # OR from template
#              [% IF c.list_roles() %][% do_something %][% END %]
#  Purpose   : Get list of all system roles
#  Arguments : NOTHING
#  Returns   : ARRAYREF OF STRINGS $roles
#  Throws    : Nothing

sub list_roles()
{
    my ($c,) = @_;
    my $roles = $c->cfg('roles');
    confess "There are no roles defined within the system" unless $roles;
    return [map { $_->{HAWAT_ATTR_ID()} } @$roles];
}

# check_roles(@)
#
#  Usage     : my $result = Hawat->check_roles(@roles); # OR
#              my $result = $c->check_roles(@roles);    # OR from template
#              [% IF c.check_roles('admin') %][% do_something %][% END %]
#  Purpose   : Check roles for current user
#  Arguments : ARRAY OF STRINGS @roles - ANDed list of required roles [MANDATORY]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#  Throws    : Nothing

sub check_roles(@)
{
    my ($c, @roles) = @_;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # Perform the check
    return $c->check_any_user_role(@roles);
}

# check_roles_for($@)
#
#  Usage     : my $result = Hawat->check_roles_for($user, @roles); # OR
#              my $result = $c->check_roles_for($user, @roles);    # OR from template
#              [% IF c.check_roles_for(user, 'admin') %][% do_something %][% END %]
#  Purpose   : Check roles for given user
#  Arguments : ARRAY OF STRINGS @roles - ANDed list of required roles [MANDATORY]
#  Returns   : INTEGER $result:
#                  1 if given user PASSED the ACL test
#                  0 if given user FAILED the ACL test
#  Throws    : Nothing

sub check_roles_for($@)
{
    my ($c, $user, @roles) = @_;

    # Perform the check
    foreach my $role (@roles) {
        return 1 if exists $user->{roles_h} and $user->{roles_h}->{$role};
    }
    return 0;
}

# check_me($)
#
#  Usage     : my $result = Hawat->check_me($id); # OR
#              my $result = $c->check_me($id);    # OR from template
#              [% IF c.check_me(id) %][% do_something %][% END %]
#  Purpose   : Check if current user has the same ID
#  Arguments : STRING $id - ID to check against [MANDATORY]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#  Throws    : Nothing

sub check_me($)
{
    my ($c, $id) = @_;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # ID must make sense
    return 0 unless $id;

    # Perform the check
    return ("$id" eq $c->get_current_user_id()) ? 1 : 0;
}

# check_membership($)
#
#  Usage     : my $result = Hawat->check_membership($group); # OR
#              my $result = $c->check_membership($group);    # OR from template
#              [% IF c.check_membership(group) %][% do_something %][% END %]
#  Purpose   : Check if current user is member of given group
#  Arguments : STRING $id - ID to check against [MANDATORY]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#  Throws    : Nothing

sub check_membership($)
{
    my ($c, $group) = @_;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # Group name must make sense
    return 0 unless $group;

    # Perform the membership check
    return (exists $c->user()->{groups_h}->{$group}) ? 1 : 0;
}

# check_management($)
#
#  Usage     : my $result = Hawat->check_management($group); # OR
#              my $result = $c->check_management($group);    # OR from template
#              [% IF c.check_management(group) %][% do_something %][% END %]
#  Purpose   : Check if current user is manager for given group
#  Arguments : STRING $id - ID to check against [MANDATORY]
#  Returns   : INTEGER $result:
#                  1 if current user PASSED the ACL test
#                  0 if current user FAILED the ACL test
#  Throws    : Nothing

sub check_management($)
{
    my ($c, $group) = @_;

    # Authorization may be turned off via configuration file
    return 1 unless $AUTHORIZATION;

    # Group name must make sense
    return 0 unless $group;

    # Lazy initialization of list of managed groups for this user
    unless (exists $c->user()->{groups_m}) {
        my $groups = $c->model(HAWAT_MODEL_GROUPS())->listManagedGroups($c->get_current_user_id);
        $c->user()->{groups_m} = { map { $_ => 1 } @$groups };
    }

    # Perform the membership check
    return (exists $c->user()->{groups_m}->{$group}) ? 1 : 0;
}

# get_memberships()
#
#  Usage     : my $result = Hawat->get_memberships(); # OR
#              my $result = $c->get_memberships();    # OR from template
#  Purpose   : Get hash of group memberships for current user
#  Arguments : None
#  Returns   : HASH REFERENCE $memberships
#  Throws    : Nothing

sub get_memberships()
{
    my ($c,) = @_;

    return {} unless $c->user_exists();

    return $c->user()->{groups_h};
}

sub generate_link($$$)
{
    my ($c, $type, $params, $icon) = @_;
    if ($type eq 'report') {
        return '<a data-toggle="tooltip" target="_blank" href="'.$c->uri_for($c->controller('report')->action_for('view'), $params->{id}).'" title="View details of report &quot;'.$params->{id}.'&quot;">'.$icon.'</a>';
    }
    elsif ($type eq 'reports') {
        my $search = { submit => "Search" };
        $search->{datefrom} = $params->{from} if $params->{from};
        $search->{dateto}   = $params->{to}   if $params->{to};
        $search->{search}   = $params->{id}   if $params->{id};
        return '<a data-toggle="tooltip" target="_blank" href="'.$c->uri_for($c->controller('report')->action_for('list'), $search).'" title="Search for related reports">'.$icon.'</a>';
    }
    elsif ($type eq 'group') {
        return '<a data-toggle="tooltip" target="_blank" href="'.$c->uri_for($c->controller('group')->action_for('view'), $params->{id}).'" title="View details of group &quot;'.$params->{id}.'&quot;">'.$icon.'</a>';
    }
    elsif ($type eq 'alerts_ip') {
        my $search = { advanced => 0, action => "Search" };
        $search->{datefrom} = $params->{from} if $params->{from};
        $search->{dateto}   = $params->{to}   if $params->{to};
        $search->{source}   = $params->{id}   if $params->{id};
        $search->{abuse}    = $params->{gid}  if $params->{gid};
        return '<a data-toggle="tooltip" target="_blank" href="'.$c->uri_for($c->controller('alert')->action_for('list'), $search).'" title="Search for related alerts">'.$icon.'</a>';
    }
    elsif ($type eq 'alerts_categ') {
        my $search = { advanced => 0, action => "Search" };
        $search->{datefrom} = $params->{from} if $params->{from};
        $search->{dateto}   = $params->{to}   if $params->{to};
        $search->{category} = $params->{id}   if $params->{id};
        $search->{abuse}    = $params->{gid}  if $params->{gid};
        return '<a data-toggle="tooltip" target="_blank" href="'.$c->uri_for($c->controller('alert')->action_for('list'), $search).'" title="Search for related alerts">'.$icon.'</a>';
    }
}

#
# See Moose::Manual::Construction(3) for details
#
sub BUILD {
    my $c = shift;

    # Detect and set 'hawat' package version
    if ($PKG_VERSION_FILE and -f $PKG_VERSION_FILE) {
        open(F, "$PKG_VERSION_FILE") or die "Can't open package version file '$PKG_VERSION_FILE': $!\n";
        my $pkg_version = <F>;
        close(F);
        $pkg_version =~ s/\s//g;
        unless ($pkg_version =~ /[0-9][-.a-zA-Z0-9]*/) {
            die "Invalid version string '$pkg_version' in package version file '$PKG_VERSION_FILE'\n";
        }
        $c->_set_pkgv($pkg_version);
        $c->log->debug("Package version: '$pkg_version'");
    }
}

# TODO: Check AJAX detection: return $this->getHeader('X-Requested-With') === 'XMLHttpRequest';

=head1 NAME

Hawat - Catalyst based application

=head1 SYNOPSIS

    script/hawat_server.pl

=head1 DESCRIPTION

[enter your description here]

=head1 SEE ALSO

L<Hawat::Controller::Root>, L<Catalyst>

=head1 AUTHOR

 Jan Mach, Radomír Orkáč
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz, radomir.orkac@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

#-------------------------------------------------------------------------------
# Alternative ACL code
#-------------------------------------------------------------------------------
#while (my ($path, $roles) = each(%$ACL_URL)) {
#   __PACKAGE__->allow_access_if( $path, sub {
#       my ( $c, $action ) = @_;
#
#       unless ( $c->_check_user_roles($roles)) {
#           die "permission denied";
#       }
#   });
#}
#sub _check_user_roles
#{
#   my ($c, $roles) = @_;
#
#   # Authorization may be turned off via configuration file
#   return 1 unless $AUTHORIZATION;
#
#   my $result = 0;
#   foreach my $role (@$roles) {
#       if ( $c->check_user_roles($role)) {
#           $result = 1; last;
#       }
#   }
#   return $result;
#}
#sub check_user_group
#{
#   my ($c, $group) = @_;
#   return 1;
#}

1;
