package Hawat::Model::Events;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use encoding::warnings;

use Carp;
use Scalar::Util 'blessed';
use HTML::Entities;
use JSON;
use Encode;
use Config::JSON;
#use Tie::Hash::Sorted;

use Data::Dumper;

use Log::Loger;
use App::Env::Scanner;

use Mentat::Core qw(:c_hawat);
use Mentat::Storage::Mongo;

use Mentat::Message::Builder::IDEA;
use Mentat::Message::Validator::IDEA;

use Mentat::Message::Formater::Helper;
use Mentat::Message::Formater::TextTemplate;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

# TODO: This is not a constant, this should be configuration, but we need some
# elegant way how to get to the configuration of another module
use constant COL_USERS => "users";

use constant TYPE_SIMPLE   => "simple";
use constant TYPE_ADVANCED => "advanced";

#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class             => 'Hawat::Model::Events',
    constructor       => 'new',
    devel             => 0,
    database_name     => 'mentat',
    collection_name   => 'events',
    template_dir      => '/etc/mentat/formater-templates',
    template_file     => '/etc/mentat/msgtmplts.json',
    schema_file_idea  => '/etc/mentat/schema-idea.json',
    severities        => ['low', 'medium', 'high', 'critical'],
    svc_helper        => '/etc/mentat/.core/svc.mentat-message-formater-helper.json',
);
has "devel"             => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"     => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "template_dir"      => (is => 'ro', isa => 'Str',  required => 1);
has "template_file"     => (is => 'ro', isa => 'Str',  required => 1);
has "schema_file_idea"  => (is => 'ro', isa => 'Str',  required => 1);
has "severities"        => (is => 'ro', isa => 'ArrayRef', required => 1);
has "db"                => (is => 'rw', isa => 'Mentat::Storage::Mongo');
has "helper"            => (is => 'rw', isa => 'Mentat::Message::Formater::Helper');
has "frme"              => (is => 'rw', isa => 'Mentat::Message::Formater::TextTemplate');
has "frma"              => (is => 'rw', isa => 'Mentat::Message::Formater::TextTemplate');
has "svc_helper"        => (is => 'ro', isa => 'Str',  required => 1);

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id'=> HAWAT_ATTR_MID(),
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc'  => 1,
        'descending' => -1,
        'ascending'  => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# _pp($)
#
#  Usage     :
#  Purpose   : Postprocess the group object when returned from database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _pp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event,) = @_;

    $event->{HAWAT_ATTR_ID()} = $event->{HAWAT_ATTR_MID()};

    return $event;
}

# _unpp($)
#
#  Usage     :
#  Purpose   : Unpostprocess the group object when updated back into database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _unpp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event,) = @_;

    delete $event->{HAWAT_ATTR_ID()};

    return $event;
}

# find($$$$$)
#
#  Usage     : my ($events, $count, $count_all, $duration, $query) = $c->model('Groups')->find($search, $limit, $skip, $sortby, $sort);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick) = @_;

    # Initialize the searching condition
    my $query = {};
    if ($search) {
        my $tmp = $search->{search};
        $query->{HAWAT_ATTR_MID()} = qr/$tmp/ if $tmp;
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($events, $count_all) = $self->db->find($query, undef, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    foreach my $g (@$events) {
        $self->_pp($g);
    }

    return ($events, scalar @$events, $count_all, $duration, $self->db->get_last_filter());
}

# fetch($)
#
#  Usage     : my ($event, $count, $duration) = $c->model('Groups')->fetch($id);
#  Purpose   : Get one particular group data within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id,) = @_;
    die "Group ID must be given as argument\n" unless $event_id;

    my $start = time();
    my ($events, $count) = $self->db->find({HAWAT_ATTR_MID() => $event_id});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $event = shift $events;
    $self->_pp($event);

    return ($event, $count, $duration);
}

# list()
#
#  Usage     : my $users = $c->model('Groups')->list();
#  Purpose   : Get the list of all groups
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub list()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();
    my $result = [];
    my ($events, $count) = $self->db->find();
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    map { push(@$result, $_->{HAWAT_ATTR_MID()}) } @$events;
    $result = [sort @$result];

    return $result;
}

# import()
#
#  Usage     : my $users = $c->model('Events')->import();
#  Purpose   : Import events from external file
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub import()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $cfg_parser = Config::JSON->new();

    # Load the templates from external file
    my ($config, $error);
    ($config, $error) = $cfg_parser->config($self->template_file);
    die 'Template file syntax error: ' . $error . "\n" unless $config;

    my $templates = $config->{templates};
    my $result = [];
    my $errors = [];
    my $item;

    foreach my $key (sort keys %$templates) {
        next unless $key;
        $item = $templates->{$key};
        $item->{HAWAT_ATTR_MID()} = $key;

        eval {
            push(@$result, $self->db->insert($item));
        };
        if ($@) {
            push(@$errors, $@);
        }
    }

    return ($result, $errors);
}

# export()
#
#  Usage     : my $users = $c->model('Events')->export();
#  Purpose   : Export events to external file
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub export()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my ($events, $count_all) = $self->db->find({'enabled' => 1});
    return '' unless $count_all;

    foreach my $e (@$events) {
        $self->_pp($e);
    }

    my $json = {
            "version"     => 1.0,
            "timestamp"   => time,
            "description" => 'Exported message templates',
            "templates"   => {},
        };
    foreach my $t (@$events) {
        $json->{templates}->{$t->{_id}} = $t;
    }

    return to_json($json, {utf8 => 1, pretty => 1});
}

# create_object($)
#
#  Usage     : my $id = $c->model('Groups')->create_object($form);
#  Purpose   : Create new group within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create_object($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form, $c) = @_;
    die "Expected event creation arguments\n" unless $form and ref $form eq 'HASH';

    my $event_object = {
            HAWAT_ATTR_MID() => $form->{'id'},
            'alias'       => $form->{'alias'},
            'title'       => $form->{'title'},
            'analyzer'    => $form->{'analyzer'},
            'class'       => $form->{'class'},
            'category'    => $form->{'category'},
            'severity'    => $form->{'severity'},
            'ref'         => $form->{'ref'},
            'enabled'     => $form->{'enabled'},
            'description' => $form->{'description'},
            'label_e'     => $form->{'label_e'},
            'label_s'     => $form->{'label_s'},
            'note'        => $form->{'note'},
            'args'        => $form->{'args'},
            'rules_idea'  => $form->{'rules_idea'},
            'cmd_example' => $form->{'cmd_example'},
            'cmd_help'    => $form->{'cmd_help'},
        };

    my @errors = $self->validate($event_object, $c);
    return ($event_object, \@errors);
}

# update_object($$)
#
#  Usage     : my $id = $c->model('Groups')->update_object($event_object, $form);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update_object($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_object, $form, $c) = @_;
    die "Event object must be given as argument\n" unless $event_object and ref $event_object eq 'HASH';
    die "Expected event form arguments\n" unless $form and ref $form eq 'HASH';

    $event_object->{'alias'}       = $form->{'alias'};
    $event_object->{'title'}       = $form->{'title'};
    $event_object->{'analyzer'}    = $form->{'analyzer'};
    $event_object->{'class'}       = $form->{'class'};
    $event_object->{'category'}    = $form->{'category'};
    $event_object->{'severity'}    = $form->{'severity'};
    $event_object->{'ref'}         = $form->{'ref'};
    $event_object->{'enabled'}     = $form->{'enabled'};
    $event_object->{'description'} = $form->{'description'};
    $event_object->{'label_e'}     = $form->{'label_e'};
    $event_object->{'label_s'}     = $form->{'label_s'};
    $event_object->{'note'}        = $form->{'note'};
    $event_object->{'args'}        = $form->{'args'};
    $event_object->{'rules_idea'}  = $form->{'rules_idea'};
    $event_object->{'cmd_example'} = $form->{'cmd_example'};
    $event_object->{'cmd_help'}    = $form->{'cmd_help'};

    my @errors = $self->validate($event_object, $c);
    return ($event_object, \@errors);
}

# create($)
#
#  Usage     : my $id = $c->model('Groups')->create($form);
#  Purpose   : Create new group within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_object, $c) = @_;
    die "Expected event object as argument\n" unless $event_object and ref $event_object eq 'HASH';

    my @errors = $self->validate($event_object, $c);

    return (undef, \@errors) if @errors;
    return $self->db->insert($event_object);
}

# update($$$)
#
#  Usage     : my $id = $c->model('Groups')->update($id, $event_object, $form);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id, $event_object, $c) = @_;
    die "Event ID must be given as argument\n" unless $event_id;
    die "Event object must be given as argument\n" unless $event_object and ref $event_object eq 'HASH';
    die "A: Catalyst must be given as argument\n" unless $c;

    my @errors = $self->validate($event_object, $c);
    return (undef, \@errors) if @errors;

    $event_object = $self->_unpp($event_object);
    return $self->db->update({HAWAT_ATTR_MID() => $event_id}, $event_object);
}

# rename($$$)
#
#  Usage     : my $id = $c->model('Groups')->rename($id, $event_object, $new_id);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub rename($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id, $event_object, $new_id, $c) = @_;
    die "Event ID must be given as argument\n" unless $event_id;
    die "Event object must be given as argument\n" unless $event_object and ref $event_object eq 'HASH';
    #die "Expected new event ID as argument\n" unless $new_id;

    # Replace the unique identifier
    $event_object->{HAWAT_ATTR_ID()}  = $new_id;
    $event_object->{HAWAT_ATTR_MID()} = $new_id;

    # Perform the validation to make sure, that we do not delete something too soon
    my @errors = $self->validate($event_object, $c);

    # Let group correct the mistakes
    return (undef, \@errors) if @errors;

    # Delete the previous record
    my $rv = $self->delete($event_id);
    die "Unable to delete previous record '$event_id'\n" unless ($rv->{ok} and $rv->{n});

    #$event_object = $self->_unpp($event_object);

    # Reinsert the modified record
    return $self->create($event_object);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Events')->delete($id);
#  Purpose   : Delete particular group from the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id,) = @_;
    die "Event ID must be given as argument\n" unless $event_id;

    return $self->db->delete($event_id);
}

# remove($)
#
#  Usage     : my $rv = $c->model('Events')->remove($id);
#  Purpose   : Delete particular group from the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub remove($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_filter,) = @_;

    return $self->db->remove($event_filter);
}

# generate_idea($)
#
#  Usage     : my $rv = $c->model('Events')->generate_idea($event);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub generate_idea($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event,) = @_;

    # Perform some sanity checks first
    unless ($event and ref $event eq 'HASH') {
        return (undef, 'Event object must be given as argument');
    }
    unless ($event->{rules_idea}) {
        return (undef, 'Event does not have the IDEA message rules defined');
    }
    unless (scalar @{$event->{rules_idea}}) {
        return (undef, 'Event does not have the IDEA message rules defined');
    }
    unless ($event->{cmd_example}) {
        return (undef, 'Event does not have the example command defined');
    }

    # Prepare the message builder configurations
    my $instance_config = {
        'name' => join('.', 'pickup', App::Env::Scanner::host_name()),
    };

    # Convert the message rules to necessary format
    my $message_rules = [];
    foreach my $r (@{$event->{rules_idea}}) {
        push(@$message_rules, [$r->{k}, $r->{v}]);
    }

    # Prepare necessary tools
    my $builder   = Mentat::Message::Builder::IDEA->new($instance_config);
    my $validator = Mentat::Message::Validator::IDEA->new($self->schema_file_idea);

    # Prepare the rules for building the message
    my @args = $builder->parse_command_args($event->{cmd_example});

    # Build and validate the example message
    my ($message, $error);
    eval {
        $message = $builder->build(rules     => $message_rules,
                                   name_args => {},
                                   pos_args  => \@args);
        $error   = $validator->validate_s($message);
    };
    if ($@) {
        return (undef, $@);
    }
    return ($message, $error);
}

# generate_email($)
#
#  Usage     : my $rv = $c->model('Events')->generate_email($event, $message);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub generate_email($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event, $message) = @_;

    my @section_ids = $self->frme->detect_section_ids($message);

    my $result = {};
    foreach my $type ('summary', 'extra') {
        $result->{$type}->{email} = $self->frme->format_section($section_ids[0], [$message], $type, 'default');
        $result->{$type}->{attch} = $self->frma->format_section($section_ids[0], [$message], $type, 'default');
    }
    return $result;
}

sub tree_view($@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($events, @rules) = @_;

    my $result = {};
    my $tmp;
    foreach my $e (@$events) {
        $tmp = $result;
        foreach my $r (@rules) {
            $tmp->{$e->{$r}} = {} unless exists $tmp->{$e->{$r}};
            $tmp = $tmp->{$e->{$r}};
        }
        $tmp->{$e->{id}} = 1;
    }
    return $result;
}

# validate($)
#
#  Usage     : my $id = $c->model('Events')->validate($event_object);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validate($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_object, $c) = @_;
    die "Event object must be given as argument\n" unless $event_object and ref $event_object eq 'HASH';
    #die "Catalyst must be given as argument\n" unless $c;

    my @errors = ();

    unless ($event_object->{HAWAT_ATTR_MID()}) {
        push(@errors, ['id','Unique event identifier is mandatory']);
    }
    unless ($event_object->{alias}) {
        push(@errors, ['alias','Unique event alias is mandatory']);
    }
    unless ($event_object->{title}) {
        push(@errors, ['title','Event title is mandatory']);
    }
    unless ($event_object->{class}) {
        push(@errors, ['class','Event classification is mandatory']);
    }
    unless ($event_object->{severity}) {
        push(@errors, ['severity','Event severity is mandatory']);
    }
    else {
        my %h = (); foreach my $s (@{$self->severities}) { $h{$s} = 1; }
        unless (exists $h{$event_object->{severity}}) {
            push(@errors, ['severity','Invalid severity value \''.$event_object->{severity}.'\', possible values are: '.join(', ', keys(%h))]);
        }
    }
    unless ($event_object->{label_e}) {
        push(@errors, ['label_e','Event label for extra report is mandatory']);
    }
    unless ($event_object->{label_s}) {
        push(@errors, ['label_s','Event label for summary report is mandatory']);
    }
    unless ($event_object->{args}) {
        push(@errors, ['args','Number of template arguments']);
    }
    unless ($event_object->{cmd_example}) {
        push(@errors, ['cmd_example', 'Event does not have the example command defined']);
    }
    unless ($event_object->{cmd_help}) {
        push(@errors, ['cmd_help', 'Event does not have the help for example command defined']);
    }
    if ($event_object->{rules_idea}) {
        unless (ref $event_object->{rules_idea}) {
            eval {
                $event_object->{rules_idea} = decode_json($event_object->{rules_idea});
            };
            if ($@) {
                push(@errors, ['rules_idea','Invalid IDEA rules JSON: '. $@]);
            }
        }
        unless (ref $event_object->{rules_idea} eq 'ARRAY') {
            push(@errors, ['rules_idea','Invalid IDEA rules format, expected ARRAY reference']);
        }
    }
    else {
        push(@errors, ['rules_idea','IDEA rules are mandatory']);
    }
    # Perform encoding into UTF8
    foreach my $k ('title', 'label_e', 'label_s', 'description', 'note') {
        if ($event_object->{$k}) {
            {
                use utf8;
                my $s;

                $c->log->info("A. For key '$k' received value: ".$event_object->{$k}."\n");

                # TESTS
                #$s = $event_object->{$k};
                #$c->log->info("B1. Value before: ".$s."\n");
                #$c->log->info("B2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #utf8::encode($s);
                #$c->log->info("B3. Value after: ".$s."\n");
                #$c->log->info("B4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                #$s = $event_object->{$k};
                #$c->log->info("C1. Value before: ".$s."\n");
                #$c->log->info("C2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #utf8::decode($s);
                #$c->log->info("C3. Value after: ".$s."\n");
                #$c->log->info("C4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                #$s = $event_object->{$k};
                #$c->log->info("D1. Value before: ".$s."\n");
                #$c->log->info("D2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #utf8::upgrade($s);
                #$c->log->info("D3. Value after: ".$s."\n");
                #$c->log->info("D4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                #$s = $event_object->{$k};
                #$c->log->info("E1. Value before: ".$s."\n");
                #$c->log->info("E2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #utf8::downgrade($s);
                #$c->log->info("E3. Value after: ".$s."\n");
                #$c->log->info("E4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                #$s = $event_object->{$k};
                #$c->log->info("F1. Value before: ".$s."\n");
                #$c->log->info("F2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #$s = encode("UTF-8", $s);
                #$c->log->info("F3. Value after: ".$s."\n");
                #$c->log->info("F4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                #$s = $event_object->{$k};
                #$c->log->info("G1. Value before: ".$s."\n");
                #$c->log->info("G2. UTF8 flag before: ".utf8::is_utf8($s)."\n");
                #$s = decode("UTF-8", $s);
                #$c->log->info("G3. Value after: ".$s."\n");
                #$c->log->info("G4. UTF8 flag after: ".utf8::is_utf8($s)."\n");
                #$c->log->info("----------------\n");
                # SOLUTION
                $s = $event_object->{$k};
                $c->log->info("H1. Before: ".$s."\n");
                $c->log->info("H2. Before flag: ".utf8::is_utf8($s)."\n");
                unless (utf8::is_utf8($s)) {
                    #utf8::decode($s);
                    $s = decode("UTF-8", $s);
                    $c->log->info("H3. Decoding: ".$s."\n");
                    $c->log->info("H4. Decoding flag: ".utf8::is_utf8($s)."\n");
                }
                $c->log->info("H5. Storing: ".$s."\n");
                $c->log->info("H6. Storing flag: ".utf8::is_utf8($s)."\n");

                #$c->log->info("Z. For key '$k' storing value: ".$s."\n");
                $event_object->{$k} = $s;
            }
        }
    }
    # Checkbox provides value 'on', if checked, but we want integers
    if ($event_object->{enabled}) {
        $event_object->{enabled} = 1;
    }
    else {
        $event_object->{enabled} = 0;
    }

    return @errors;
}

# validate_template(;$)
#
#  Usage     : my $id = $c->model('Events')->validate_template();
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validate_template(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @event_objects = @_;

    my $result = [];
    my $errors = [];

    my ($example, $error);
    foreach my $eo (@event_objects) {
        ($example, $error) = $self->generate_idea($eo);
        unless ($error) {
            push (@$result, "Valid IDEA template for $eo->{_id}");
        }
        else {
            push (@$result, "Invalid IDEA template for $eo->{_id}: '$error'");
        }
    }

    return ($result, $errors);
}

# enable($$$)
#
#  Usage     : my $rv = $c->model('Groups')->enable($group_id, $filter_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub enable($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id,) = @_;
    die "Event ID must be given as argument\n" unless $event_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $event_id},
                { '$set' => { 'enabled' => 1 }},
            );
}

# disable($$$)
#
#  Usage     : my $rv = $c->model('Groups')->disable($group_id, $filter_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub disable($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event_id,) = @_;
    die "Event ID must be given as argument\n" unless $event_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $event_id},
                { '$set' => { 'enabled' => 0 }},
            );
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal helper objects for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name)
        );
    $self->helper(Mentat::Message::Formater::Helper->service($self->svc_helper));
    $self->frme(Mentat::Message::Formater::TextTemplate->new($self->template_dir . '/email',
                                                             $self->helper,
            )
        );
    $self->frma(Mentat::Message::Formater::TextTemplate->new($self->template_dir . '/csv',
                                                             $self->helper,
            )
        );

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

# _decode($)
#
#  Usage     :
#  Purpose   : Postprocess the group object when returned from database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _decode($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($event,) = @_;

    $event->{HAWAT_ATTR_ID()} = $event->{HAWAT_ATTR_MID()};

    return $event;
}

1;
