# db.groups.update({}, {$rename:{'manager': 'managers'}},{'multi':1})
#
package Hawat::Model::Groups;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use HTML::Entities;
use Data::Dumper;
use JSON;

use Log::Loger;
use Mentat::Core qw(:c_hawat);
use Mentat::Storage::Mongo;
use Mentat::Message::Storage::Mongo;
use Data::Filter;
use Mentat::Message::Value::IDEA;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

# TODO: This is not a constant, this should be configuration, but we need some
# elegant way how to get to the configuration of another module
use constant COL_USERS => "users";

use constant TYPE_SIMPLE   => "simple";
use constant TYPE_ADVANCED => "advanced";

#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class            => 'Hawat::Model::Groups',
    constructor      => 'new',
    devel            => 0,
    database_name    => 'mentat',
    collection_name  => 'groups',
    sdatabase_name   => 'mentat_stats',
    scollection_name => 'statistics',
    filter_grammar   => '/etc/mentat/.core/filter-grammar.conf',
    user_map         => {},
    default_roles    => [],
    cache_dir        => '/var/mentat/cache',
    cache_ttl        => 60*60*24*7,
);
has "devel"            => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"    => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name"  => (is => 'ro', isa => 'Str',  required => 1);
has "sdatabase_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "scollection_name" => (is => 'ro', isa => 'Str',  required => 1);
has "filter_grammar"   => (is => 'ro', isa => 'Str',  required => 1);
has "cache_dir"        => (is => 'ro', isa => 'Str',  required => 1);
has "cache_ttl"        => (is => 'ro', isa => 'Int',  required => 1);
has "db"               => (is => 'rw', isa => 'Mentat::Storage::Mongo');
has "ffactory"         => (is => 'rw', isa => 'Data::Filter::Factory');
has "aggregator"       => (is => 'rw', isa => 'Mentat::Storage::Mongo::Aggregator');

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id'=> HAWAT_ATTR_MID(),
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc'  => 1,
        'descending' => -1,
        'ascending'  => 1,
    );

my $PERIODS = [
    'none',
    '10mins',
    '15mins',
    '20mins',
    '30mins',
    '1hr',
    '2hrs',
    '3hrs',
    '4hrs',
    '6hrs',
    '8hrs',
    '12hrs',
    '1day',
    '2days',
    '3days',
    '4days',
    '5days',
    '6days',
    '7days',
    '8days',
    '9days',
    '10days',
    '1week',
    '2weeks',
    '3weeks',
    '4weeks',
];

my $PERIODS_H = {
    'none'   => 0,
    '10mins' => 600,
    '15mins' => 900,
    '20mins' => 1200,
    '30mins' => 1800,
    '1hr'    => 3600,
    '2hrs'   => (3600 * 2),
    '3hrs'   => (3600 * 3),
    '4hrs'   => (3600 * 4),
    '6hrs'   => (3600 * 6),
    '8hrs'   => (3600 * 8),
    '12hrs'  => (3600 * 12),
    '1day'   => 86400,
    '2days'  => (86400 * 2),
    '3days'  => (86400 * 3),
    '4days'  => (86400 * 4),
    '5days'  => (86400 * 5),
    '6days'  => (86400 * 6),
    '7days'  => (86400 * 7),
    '8days'  => (86400 * 8),
    '9days'  => (86400 * 9),
    '10days' => (86400 * 10),
    '1week'  => 604800,
    '2weeks' => (604800 * 2),
    '3weeks' => (604800 * 3),
    '4weeks' => (604800 * 4),
};

my $TIMING_THRS = {
    'low'      => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '1day' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'medium'   => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '1day' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'high'     => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '4hrs' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'critical' => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '10mins' }, 'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
};

my $TIMING_DFLT = {
    'low'      => { 'period' =>  '1day',   'threshold' =>  '7days', 'relapse' =>  '2days' },
    'medium'   => { 'period' =>  '2hrs',   'threshold' =>  '2days', 'relapse' =>  '1day' },
    'high'     => { 'period' =>  '10mins', 'threshold' =>  '2hrs',  'relapse' =>  'none' },
    'critical' => { 'period' =>  '10mins', 'threshold' =>  'none',  'relapse' =>  'none' },
};

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

sub get_timing_consts()
{
    my $self = shift;
    return (periods     => $PERIODS,
            periods_h   => $PERIODS_H,
            timing_thrs => $TIMING_THRS,
            timing_dflt => $TIMING_DFLT);
}

sub get_item_sets($$)
{
    my $self = shift;
    my $rewrite = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->aggregator->get_item_sets((!defined $rewrite)?0:$rewrite);
}

# _pp($)
#
#  Usage     :
#  Purpose   : Postprocess the group object when returned from database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _pp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group,) = @_;

    $group->{id} = $group->{HAWAT_ATTR_MID()};
    map { $group->{managers_h}->{$_->{id}} = $_->{id}; } @{$group->{HAWAT_ATTR_MANAGERS()}};

    return $group;
}

# _unpp($)
#
#  Usage     :
#  Purpose   : Unpostprocess the group object when updated back into database
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _unpp($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group,) = @_;

    delete $group->{HAWAT_ATTR_ID()};
    delete $group->{managers_h};

    return $group;
}

# find($$$$$)
#
#  Usage     : my ($groups, $count, $count_all, $duration, $query) = $c->model('Groups')->find($search, $limit, $skip, $sortby, $sort);
#  Purpose   : Get list of all reports according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick) = @_;

    # Initialize the searching condition
    my $query = {};
    if ($search) {
        my $tmp = $search->{search};
        $query->{HAWAT_ATTR_MID()} = qr/$tmp/ if $tmp;

        $query->{ts_from} = {'$lte' => Value::Convertor->datestr_to_unixs($search->{to})}   if $search->{to};
        $query->{ts_to}   = {'$gte' => Value::Convertor->datestr_to_unixs($search->{from})} if $search->{from};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($groups, $count_all) = $self->db->find($query, undef, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    foreach my $g (@$groups) {
        $self->_pp($g);
    }

    return ($groups, scalar @$groups, $count_all, $duration, $self->db->get_last_filter());
}

# fetch($)
#
#  Usage     : my ($group, $count, $duration) = $c->model('Groups')->fetch($id);
#  Purpose   : Get one particular group data within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $start = time();
    my ($groups, $count) = $self->db->find({HAWAT_ATTR_MID() => $group_id});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $group = shift $groups;
    $self->_pp($group);

    return ($group, $count, $duration);
}

# fetchSubitem($$$)
#
#  Usage     : my $group, $count, $duration) = $c->model('Groups')->fetchSubitem($item, $type, $id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetchSubitem($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group, $type, $id) = @_;
    die "Group must be given as argument\n" unless $group;
    die "Subitem type must be given as argument\n" unless $type;
    die "Subitem ID must be given as argument\n" unless $id;

    my %trt = ('filter' => 'rep_filters', 'network' => 'networks');
    $type = ($trt{$type})?$trt{$type}:$type;

    foreach my $i (@{$group->{$type}}) {
        return $i if $i->{HAWAT_ATTR_MID()} and $i->{HAWAT_ATTR_MID()} eq $id;
    }
    return undef;
}

# list()
#
#  Usage     : my $users = $c->model('Groups')->list();
#  Purpose   : Get the list of all groups
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub list()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $start = time();
    my $result = [];
    my ($groups, $count) = $self->db->find();
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    map { push(@$result, $_->{HAWAT_ATTR_MID()}) } @$groups;
    $result = [sort @$result];

    return $result;
}

# listManagers($)
#
#  Usage     : my $groups = $c->model('Groups')->listManagers($group_id);
#  Purpose   : Get the list of managers managing this particular group
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listManagers($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $start = time();
    my $result = [];
    my ($groups, $count) = $self->db->find({HAWAT_ATTR_MID() => $group_id});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    my $group = shift $groups;
    map { push(@$result, $_->{id}) } @{$group->{HAWAT_ATTR_MANAGERS()}};
    $result = [sort @$result];

    return $result;
}

# listManagedGroups($)
#
#  Usage     : my $groups = $c->model('Groups')->listManagedGroups($user_id);
#  Purpose   : Get the list of groups managed by particular user
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listManagedGroups($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    my $start = time();
    my $result = [];
    my ($groups, $count) = $self->db->find(
                                { HAWAT_ATTR_MANAGERS().'.$id' => $user_id },
                                { HAWAT_ATTR_MID() => 1 },
                                { HAWAT_ATTR_MID() => 1 });
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return $result unless $count;

    map { push(@$result, $_->{HAWAT_ATTR_MID()}) } @$groups;
    $result = [sort @$result];

    return $result;
}

# listFilters($)
#
#  Usage     : my ($filters, $count, $count_all, $duration, $query) = $c->model('Groups')->listFilters($search, $limit, $skip, $sortby, $sort);
#  Purpose   : Get the list of filters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listFilters($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort) = @_;

    # Initialize the searching condition
    if ($search) {
        $search = { '$or' => [{HAWAT_ATTR_MID() => qr/$search/}, {'rep_filters._id' => qr/$search/}]};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($groups, $count_g) = $self->db->find($search, undef, $sort_spec);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_g;

    my $result    = [];
    my $count     = 0;
    my $count_all = 0;

    foreach my $g (@$groups) {
        if ($g->{rep_filters}) {
            my $tmp = {i => $g->{_id}, f => []};
            foreach my $f (@{$g->{rep_filters}}) {
                ++$count_all;
                if ($skip > 0) {
                    --$skip; next;
                }

                next if ($limit and $count > $limit);

                ++$count;
                push(@{$tmp->{f}}, $f);
            }
            push(@$result, $tmp) if scalar @{$tmp->{f}};
        }
    }

    $duration = Value::Convertor->duration_to_string(time() - $start);
    return ($result, $count, $count_all, $duration, $self->db->get_last_filter());
}

# listGroupFilters($)
#
#  Usage     : my ($filters, $count) = $c->model('Groups')->listGroupFilters($gid);
#  Purpose   : Get the list of filters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listGroupFilters($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($gid) = @_;

    my ($group, $count, $duration) = $self->fetch($gid);

    my $filters = []; my $cnt = 0;
    if ($group and $group->{rep_filters}) {
        foreach my $f (@{$group->{rep_filters}}) {
            ++$cnt;
            push(@$filters, $f);
        }
    }
    return ($filters, $cnt);
}

# listNetworks($)
#
#  Usage     : my ($filters, $count, $count_all, $duration, $query) = $c->model('Groups')->listNetworks($group_id, $limit, $skip, $sortby, $sort);
#  Purpose   : Get the list of networks
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listNetworks($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort) = @_;

    # Initialize the searching condition
    if ($search) {
        $search = { '$or' => [
                {HAWAT_ATTR_MID()   => qr/$search/},
                {'networks._id'     => qr/$search/},
                {'networks.netname' => qr/$search/},
                {'networks.network' => qr/$search/},
            ]};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $start = time();
    my ($groups, $count_g) = $self->db->find($search, undef, $sort_spec);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ([], 0, 0, $duration, $self->db->get_last_filter()) unless $count_g;

    my $result    = [];
    my $count     = 0;
    my $count_all = 0;

    foreach my $g (@$groups) {
        if ($g->{networks}) {
            my $tmp = {i => $g->{_id}, n => []};
            foreach my $n (@{$g->{networks}}) {
                ++$count_all;
                if ($skip > 0) {
                    --$skip; next;
                }

                next if ($limit and $count >= $limit);

                ++$count;
                push(@{$tmp->{n}}, $n);
            }
            push(@$result, $tmp) if scalar @{$tmp->{n}};
        }
    }

    $duration = Value::Convertor->duration_to_string(time() - $start);
    return ($result, $count, $count_all, $duration, $self->db->get_last_filter());
}

# listGroupNetworks($)
#
#  Usage     : my ($filters, $count) = $c->model('Groups')->listGroupNetworks($gid);
#  Purpose   : Get the list of networks
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub listGroupNetworks($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($gid) = @_;

    my ($group, $count, $duration) = $self->fetch($gid);

    my $networks = []; my $cnt = 0;
    if ($group and $group->{networks}) {
        foreach my $f (@{$group->{networks}}) {
            ++$cnt;
            push(@$networks, $f);
        }
    }
    return ($networks, $cnt);
}

# create_object($)
#
#  Usage     : my $id = $c->model('Groups')->create_object($form);
#  Purpose   : Create new group within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create_object($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form,) = @_;
    die "Expected group creation arguments\n" unless $form and ref $form eq 'HASH';

    my $group_object = {
            HAWAT_ATTR_MID()      => $form->{'id'},
            HAWAT_ATTR_DESC()     => $form->{'description'},
            HAWAT_ATTR_SOURCE()   => $form->{'source'},
            HAWAT_ATTR_TS()       => time,
            HAWAT_ATTR_MANAGERS() => [],
            HAWAT_ATTR_QUERIES()  => [],
            'rep_mode'        => $form->{'rep_mode'},
            'rep_template'    => $form->{'rep_template'},
            'rep_attach'      => $form->{'rep_attach'},
            'rep_emails'      => $form->{'rep_emails'},
            'rep_redirect'    => ($form->{'rep_redirect'} or 0),
            'rep_mute'        => ($form->{'rep_mute'} or 0),
            'rep_compress'    => ($form->{'rep_compress'} or 0),
            'rep_filters'     => [],
            'rep_timing'      => {},
            'rep_timing_type' => ($form->{'rep_timing_type'} or 'default'),
        };

    $group_object->{'rep_timing'}->{'low'}->{'period'}    = $form->{'rep_timing_low_period'}    if ($form->{'rep_timing_low_period'});
    $group_object->{'rep_timing'}->{'low'}->{'threshold'} = $form->{'rep_timing_low_threshold'} if ($form->{'rep_timing_low_threshold'});
    $group_object->{'rep_timing'}->{'low'}->{'relapse'}   = $form->{'rep_timing_low_relapse'}   if ($form->{'rep_timing_low_relapse'});

    $group_object->{'rep_timing'}->{'medium'}->{'period'}    = $form->{'rep_timing_medium_period'}    if ($form->{'rep_timing_medium_period'});
    $group_object->{'rep_timing'}->{'medium'}->{'threshold'} = $form->{'rep_timing_medium_threshold'} if ($form->{'rep_timing_medium_threshold'});
    $group_object->{'rep_timing'}->{'medium'}->{'relapse'}   = $form->{'rep_timing_medium_relapse'}   if ($form->{'rep_timing_medium_relapse'});

    $group_object->{'rep_timing'}->{'high'}->{'period'}    = $form->{'rep_timing_high_period'}    if ($form->{'rep_timing_high_period'});
    $group_object->{'rep_timing'}->{'high'}->{'threshold'} = $form->{'rep_timing_high_threshold'} if ($form->{'rep_timing_high_threshold'});
    $group_object->{'rep_timing'}->{'high'}->{'relapse'}   = $form->{'rep_timing_high_relapse'}   if ($form->{'rep_timing_high_relapse'});

    $group_object->{'rep_timing'}->{'critical'}->{'period'}    = $form->{'rep_timing_critical_period'}    if ($form->{'rep_timing_critical_period'});
    $group_object->{'rep_timing'}->{'critical'}->{'threshold'} = $form->{'rep_timing_critical_threshold'} if ($form->{'rep_timing_critical_threshold'});
    $group_object->{'rep_timing'}->{'critical'}->{'relapse'}   = $form->{'rep_timing_critical_relapse'}   if ($form->{'rep_timing_critical_relapse'});

    my @errors = $self->validate($group_object);
    return ($group_object, \@errors);
}

# update_object($$)
#
#  Usage     : my $id = $c->model('Groups')->update_object($group_object, $form);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update_object($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_object, $form) = @_;
    die "Group object must be given as argument\n" unless $group_object and ref $group_object eq 'HASH';
    die "Expected group creation arguments\n" unless $form and ref $form eq 'HASH';

    $group_object->{HAWAT_ATTR_DESC()}   = $form->{'description'};
    $group_object->{HAWAT_ATTR_SOURCE()} = $form->{'source'};
    $group_object->{'rep_mode'}        = $form->{'rep_mode'};
    $group_object->{'rep_template'}    = $form->{'rep_template'};
    $group_object->{'rep_attach'}      = $form->{'rep_attach'};
    $group_object->{'rep_emails'}      = $form->{'rep_emails'};
    $group_object->{'rep_timing'}      = ($form->{'rep_timing'} or {});
    $group_object->{'rep_redirect'}    = ($form->{'rep_redirect'} or 0);
    $group_object->{'rep_mute'}        = ($form->{'rep_mute'} or 0);
    $group_object->{'rep_compress'}    = ($form->{'rep_compress'} or 0);
    $group_object->{'rep_timing_type'} = ($form->{'rep_timing_type'} or 'default');

    $group_object->{'rep_timing'}->{'low'}->{'period'}    = $form->{'rep_timing_low_period'}    if ($form->{'rep_timing_low_period'});
    $group_object->{'rep_timing'}->{'low'}->{'threshold'} = $form->{'rep_timing_low_threshold'} if ($form->{'rep_timing_low_threshold'});
    $group_object->{'rep_timing'}->{'low'}->{'relapse'}   = $form->{'rep_timing_low_relapse'}   if ($form->{'rep_timing_low_relapse'});

    $group_object->{'rep_timing'}->{'medium'}->{'period'}    = $form->{'rep_timing_medium_period'}    if ($form->{'rep_timing_medium_period'});
    $group_object->{'rep_timing'}->{'medium'}->{'threshold'} = $form->{'rep_timing_medium_threshold'} if ($form->{'rep_timing_medium_threshold'});
    $group_object->{'rep_timing'}->{'medium'}->{'relapse'}   = $form->{'rep_timing_medium_relapse'}   if ($form->{'rep_timing_medium_relapse'});

    $group_object->{'rep_timing'}->{'high'}->{'period'}    = $form->{'rep_timing_high_period'}    if ($form->{'rep_timing_high_period'});
    $group_object->{'rep_timing'}->{'high'}->{'threshold'} = $form->{'rep_timing_high_threshold'} if ($form->{'rep_timing_high_threshold'});
    $group_object->{'rep_timing'}->{'high'}->{'relapse'}   = $form->{'rep_timing_high_relapse'}   if ($form->{'rep_timing_high_relapse'});

    $group_object->{'rep_timing'}->{'critical'}->{'period'}    = $form->{'rep_timing_critical_period'}    if ($form->{'rep_timing_critical_period'});
    $group_object->{'rep_timing'}->{'critical'}->{'threshold'} = $form->{'rep_timing_critical_threshold'} if ($form->{'rep_timing_critical_threshold'});
    $group_object->{'rep_timing'}->{'critical'}->{'relapse'}   = $form->{'rep_timing_critical_relapse'}   if ($form->{'rep_timing_critical_relapse'});

    my @errors = $self->validate($group_object);
    return ($group_object, \@errors);
}

# create($)
#
#  Usage     : my $id = $c->model('Groups')->create($form);
#  Purpose   : Create new group within the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_object,) = @_;
    die "Expected group object as argument\n" unless $group_object and ref $group_object eq 'HASH';

    my @errors = $self->validate($group_object);
    return (undef, \@errors) if @errors;

    return $self->db->insert($group_object);
}

# update($$)
#
#  Usage     : my $id = $c->model('Groups')->update($id, $group_object);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub update($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $group_object) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Group object must be given as argument\n" unless $group_object and ref $group_object eq 'HASH';

    my @errors = $self->validate($group_object);
    return (undef, \@errors) if @errors;

    $group_object = $self->_unpp($group_object);
    return $self->db->update({HAWAT_ATTR_MID() => $group_id}, $group_object);
}

# rename($$$)
#
#  Usage     : my $id = $c->model('Groups')->rename($id, $group_object, $new_id);
#  Purpose   : Create new account within the account storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub rename($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $group_object, $new_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Group object must be given as argument\n" unless $group_object and ref $group_object eq 'HASH';
    #die "Expected new group ID as argument\n" unless $new_id;

    # Replace the unique identifier
    $group_object->{HAWAT_ATTR_ID()}  = $new_id;
    $group_object->{HAWAT_ATTR_MID()} = $new_id;

    # Perform the validation to make sure, that we do not delete something too soon
    my @errors = $self->validate($group_object);

    # Let group correct the mistakes
    return (undef, \@errors) if @errors;

    # Delete the previous record
    my $rv = $self->delete($group_id);
    die "Unable to delete previous record '$group_id'\n" unless ($rv->{ok} and $rv->{n});

    $group_object = $self->_unpp($group_object);

    # Reinsert the modified record
    return $self->create($group_object);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Groups')->delete($id);
#  Purpose   : Delete particular group from the group storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    return $self->db->delete($group_id);
}

# addManager($$)
#
#  Usage     : my $rv = $c->model('Users')->addManager($group_id, $user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addManager($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $user_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "User ID must be given as argument\n" unless $user_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    HAWAT_ATTR_MANAGERS() => MongoDB::DBRef->new(
                        db => $self->database_name,
                        ref => COL_USERS(),
                        id => $user_id)
                }}
            );
}

# removeManager($$)
#
#  Usage     : my $rv = $c->model('Users')->removeManager($group_id, $user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeManager($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $user_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "User ID must be given as argument\n" unless $user_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    HAWAT_ATTR_MANAGERS() => { '$id' => $user_id }
                }}
            );
}

# deleteManagerRefs($)
#
#  Usage     : my $rv = $c->model('Groups')->deleteManagerRefs($user_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub deleteManagerRefs($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($user_id,) = @_;
    die "User ID must be given as argument\n" unless $user_id;

    return $self->db->update(
                { HAWAT_ATTR_MANAGERS().'.$id' => $user_id },
                { '$pull' => {
                    HAWAT_ATTR_MANAGERS() => { '$id' => $user_id }
                }},
                { 'multiple' => 1 },
            );
}

# addQuery($$)
#
#  Usage     : my $rv = $c->model('Groups')->addQuery($group_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $query) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Query must be given as argument\n" unless $query;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    HAWAT_ATTR_QUERIES() => $query,
                }}
            );
}

# removeQuery($$)
#
#  Usage     : my $rv = $c->model('Groups')->removeQuery($group_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $query_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Query ID must be given as argument\n" unless $query_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    HAWAT_ATTR_QUERIES() => { HAWAT_ATTR_MID() => $query_id }
                }}
            );
}

# getQuery($$)
#
#  Usage     : my $rv = $c->model('Groups')->getQuery($group_id, $query_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub getQuery($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $query_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Query ID must be given as argument\n" unless $query_id;

    my ($groups, $count) = $self->db->find(
            { HAWAT_ATTR_MID() => $group_id, HAWAT_ATTR_QUERIES().".".HAWAT_ATTR_MID() => $query_id },
            { HAWAT_ATTR_QUERIES() => 1});
    return undef unless $count;

    my $group = shift $groups;
    foreach my $query (@{$group->{HAWAT_ATTR_QUERIES()}}) {
        return $query if $query_id eq $query->{HAWAT_ATTR_MID()};
    }
    die "Something weird happened, the query '$query_id' should be in storage, but it isn't\n";
}

# create_filter($$)
#
#  Usage     : my $id = $c->model('Groups')->create_filter($form, $eventdb);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create_filter($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form, $eventdb) = @_;
    die "Expected group creation arguments\n" unless $form and ref $form eq 'HASH';

    my $filter = {
            HAWAT_ATTR_MID()  => $form->{id},
            HAWAT_ATTR_DESC() => $form->{description},
            'ts'              => time(),
            'note'            => $form->{note},
            'analyzers'       => $form->{analyzers},
            'categories'      => $form->{categories},
            'ips'             => $form->{ips},
            'filter'          => $form->{filter},
            'enabled'         => $form->{enabled},
            'type'            => $form->{type},
            'validfrom'       => $form->{validfrom},
            'validto'         => $form->{validto},
        };
    my @errors = $self->validateFilter($filter, $eventdb);
    return ($filter, \@errors);
}

# addFilter($$$)
#
#  Usage     : my $rv = $c->model('Groups')->addFilter($group_id, $form, $eventdb);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addFilter($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $filter, $eventdb) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Expected filter as argument\n" unless $filter and ref $filter eq 'HASH';

    # Let group correct the mistakes
    my @errors = $self->validateFilter($filter, $eventdb);
    return (undef, \@errors) if @errors;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    'rep_filters' => $filter,
                }},
            );
}

# updateFilter($$$)
#
#  Usage     : my $rv = $c->model('Groups')->updateFilter($group_id, $filter_id, $form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub updateFilter($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $filter_id, $filter, $eventdb) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Filter ID must be given as argument\n" unless $filter_id;
    die "Expected filter as argument\n" unless $filter and ref $filter eq 'HASH';

    # Let group correct the mistakes
    my @errors = $self->validateFilter($filter, $eventdb);
    return (undef, \@errors) if @errors;

    # Remove the existing filter
    # TODO: solve the error checking
    $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    'rep_filters' => { HAWAT_ATTR_MID() => $filter_id }
                }}
            );
    # Insert the new one
    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    'rep_filters' => $filter,
                }},
            );
}

# removeFilter($$$)
#
#  Usage     : my $rv = $c->model('Groups')->updateFilter($group_id, $filter_id, $form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeFilter($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $filter_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Filter ID must be given as argument\n" unless $filter_id;

    # Remove the existing filter
    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    'rep_filters' => { HAWAT_ATTR_MID() => $filter_id }
                }}
            );
}

# enableFilter($$$)
#
#  Usage     : my $rv = $c->model('Groups')->enableFilter($group_id, $filter_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub enableFilter($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $filter_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Filter ID must be given as argument\n" unless $filter_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id, 'rep_filters.'.HAWAT_ATTR_MID() => $filter_id},
                { '$set' => { 'rep_filters.$.enabled' => 1 }},
            );
}

# disableFilter($$$)
#
#  Usage     : my $rv = $c->model('Groups')->disableFilter($group_id, $filter_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub disableFilter($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $filter_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Filter ID must be given as argument\n" unless $filter_id;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id, 'rep_filters.'.HAWAT_ATTR_MID() => $filter_id},
                { '$set' => { 'rep_filters.$.enabled' => 0 }},
            );
}

# test_filters($$)
#
#  Usage     : my $rv = $c->model('Groups')->test_filters($group_id, $message_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub test_filters($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $message) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Message must be given as argument\n" unless $message;

    my ($filters, $count) = $self->listGroupFilters($group_id);
    my $ts = time();
    
    if ($count) {
        my $args = {
            'compile' => 'idea',
            'order' => 'deny',
            'deny' => [],
            'default_action' => 'allow',
        };
        foreach my $f (@$filters) {
            next unless ($f->{enabled});
            next if ($f->{validfrom} and int($f->{validfrom}) and int($f->{validfrom}) > $ts);
            next if ($f->{validto} and int($f->{validto}) and int($f->{validto}) < $ts);
            push(@{$args->{deny}}, $f->{filter});
        }

        my $filter = Data::Filter->new(filter_grammar_file => $self->filter_grammar,
                                       filter_config => $args,
                                       dt_resolver => Mentat::Message::Value::IDEA->new());

        my ($result, $rule, $idx) = $filter->filter($message);

        return { 'str' => 'Filtering successfull', 'result' => $result, 'rule' => $rule, 'idx' => $idx, 'filters_all' => $filters, 'filter' => $filter };
    }

    return { 'str' => "There are no filter rules defined for group '$group_id'", 'result' => 1 };
}

# create_network($)
#
#  Usage     : my $id = $c->model('Groups')->create_network($form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub create_network($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form, $source) = @_;
    die "Expected group creation arguments\n" unless $form and ref $form eq 'HASH';

    $source = 'manual' unless $source;

    my $network = {
            HAWAT_ATTR_MID() => Mentat::Core::uid(),
            'network'        => $form->{'network'},
            'netname'        => $form->{'netname'},
            'description'    => $form->{'description'},
            #'local_abuses'   => $form->{'local_abuses'},
            'source'         => $source,
        };

    my @errors = $self->validateNetwork($network);
    return ($network, \@errors);
}

# addNetwork($$)
#
#  Usage     : my $rv = $c->model('Groups')->addNetwork($group_id, $network);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addNetwork($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $network) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Expected network creation arguments\n" unless $network and ref $network eq 'HASH';

    # Let group correct the mistakes
    my @errors = $self->validateNetwork($network);
    return (undef, \@errors) if @errors;

    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    'networks' => $network,
                }},
            );
}

# addNetworks($)
#
#  Usage     : my $rv = $c->model('Groups')->addNetworks($form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub addNetworks($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form,) = @_;
    die "Expected network creation arguments\n" unless $form and ref $form eq 'HASH';

    # Let group correct the mistakes
    my @errors = $self->validateNetworkBulk($form);
    return (undef, \@errors) if @errors;

    my @networks = split(/[,;\s]+/, $form->{networks});
    my $results = [];
    foreach my $net (@networks) {
        $net =~ s/\s//g;
        next unless $net;
        my $data = {
                'network'         => $net,
                'netname'         => $form->{'netname'},
                'description'     => $form->{'description'},
            };
        my ($network_object, $err1) = $self->create_network($data, 'bulk');
        if ($network_object) {
            my ($result, $err2) = $self->addNetwork($form->{'abuse_group'}, $network_object);
            if ($result) {
                push(@$results, ["success", "Successfully added network '$net' for abuse group '$form->{abuse_group}'"]);
            }
            else {
                push(@$results, ["error", "Unable to add network '$net' for abuse group '$form->{abuse_group}': ".Dumper($err2)]);
            }
        }
        else {
            push(@$results, ["error", "Unable to add network '$net' for abuse group '$form->{abuse_group}': ".Dumper($err1)]);
        }
    }
    return $results;
}

# updateNetwork($$$)
#
#  Usage     : my $rv = $c->model('Groups')->updateNetwork($group_id, $network_id, $form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub updateNetwork($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $network_id, $network) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Network ID must be given as argument\n" unless $network_id;
    die "Expected network creation arguments\n" unless $network and ref $network eq 'HASH';

    # Let group correct the mistakes
    my @errors = $self->validateNetwork($network);
    return (undef, \@errors) if @errors;

    # Remove the existing network
    # TODO: solve the error checking
    $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    'networks' => { HAWAT_ATTR_MID() => $network_id }
                }}
            );
    # Insert the new one
    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$addToSet' => {
                    'networks' => $network,
                }},
            );
}

# removeNetwork($$)
#
#  Usage     : my $rv = $c->model('Groups')->removeNetwork($group_id, $network_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeNetwork($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, $network_id) = @_;
    die "Group ID must be given as argument\n" unless $group_id;
    die "Network ID must be given as argument\n" unless $network_id;

    # Remove the existing filter
    return $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$pull' => {
                    'networks' => { HAWAT_ATTR_MID() => $network_id }
                }}
            );
}

# removeNetworks(;$$)
#
#  Usage     : my $rv = $c->model('Groups')->removeNetworks($group_id, $network_source);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeNetworks(;$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form,) = @_;
    die "Expected network creation arguments\n" unless $form and ref $form eq 'HASH';

    my $gid = $form->{abuse_group};
    my $ntp = $form->{network_type};
    my $results = [];

    # Remove network information from all records
    if (not $gid and not $ntp) {
        my $rv = $self->db->update(
                { },
                { '$set' => { 'networks' => [] } },
                { 'multiple' => 1 }
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            push(@$results, ["success", "Successfully removed all network information from all '$rv->{n}' groups"]);
        }
        else {
            push(@$results, ["error", "Unable to remove all network information from all groups"]);
        }
    }
    elsif ($gid and not $ntp) {
        my $rv = $self->db->update(
                { HAWAT_ATTR_MID() => $gid },
                { '$set' => { 'networks' => [] } },
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            push(@$results, ["success", "Successfully removed all network information from '$gid' group"]);
        }
        else {
            push(@$results, ["error", "Unable to remove all network information from '$gid' group"]);
        }
    }
    elsif (not $gid and $ntp) {
        my $rv = $self->db->update(
                {},
                { '$pull' => { 'networks' => { 'source' => $ntp } } },
                { 'multiple' => 1 }
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            push(@$results, ["success", "Successfully removed '$ntp' network information from '$rv->{n}' groups"]);
        }
        else {
            push(@$results, ["error", "Unable to remove '$ntp' network information from all groups"]);
        }
    }
    else {
        my $rv = $self->db->update(
                { HAWAT_ATTR_MID() => $gid },
                { '$pull' => { 'networks' => { 'source' => $ntp } } },
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            push(@$results, ["success", "Successfully removed '$ntp' network information from '$gid' group"]);
        }
        else {
            push(@$results, ["error", "Unable to remove '$ntp' network information from '$gid' group"]);
        }
    }

    return $results;
}

# check_networks($$)
#
#  Usage     : my $rv = $c->model('Groups')->check_networks($group, $network_list);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub check_networks($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group, $network_list) = @_;
    die "Group must be given as argument\n" unless $group;
    die "Network list must be given as argument\n" unless $network_list;

    # Make map of desired networks
    my %desired = ();
    foreach my $n (split(/[,;\n\t]+/, $network_list)) {
        $n =~ s/ //g;
        next unless $n;
        next if $n =~ /^\s*$/;
        $desired{$n}++;
    }

    # Make map of currently existing networks
    my %existing = ();
    foreach my $n (@{$group->{networks}}) {
        next unless $n;
        next if $n =~ /^\s*$/;
        $existing{$n->{network}}++;
    }

    my %missing = ();
    foreach my $n (keys %desired) {
        $n =~ s/[ \n\t]//g;
        next unless $n;
        next if $n =~ /^\s*$/;
        if (exists $existing{$n}) {
            delete $existing{$n};
        }
        else {
            $missing{$n}++;
        }
    }
    
    return {missing => \%missing, extra => \%existing};
}

# mute($)
#
#  Usage     : my $rv = $c->model('Groups')->mute($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub mute($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_mute' => 1 }},
                { 'multiple' => 1 }
            );
}


# unmute($)
#
#  Usage     : my $rv = $c->model('Groups')->unmute($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub unmute($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_mute' => 0 }},
                { 'multiple' => 1 }
            );
}

# redirect($)
#
#  Usage     : my $rv = $c->model('Groups')->redirect($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub redirect($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_redirect' => 1 }},
                { 'multiple' => 1 }
            );
}


# unredirect($)
#
#  Usage     : my $rv = $c->model('Groups')->unredirect($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub unredirect($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_redirect' => 0 }},
                { 'multiple' => 1 }
            );
}

# set_compression($)
#
#  Usage     : my $rv = $c->model('Groups')->set_compression($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub set_compression($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_compress' => 1 }},
                { 'multiple' => 1 }
            );
}


# unset_compression($)
#
#  Usage     : my $rv = $c->model('Groups')->unset_compression($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub unset_compression($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id,) = @_;
    die "Group ID must be given as argument\n" unless $group_id;

    my $flt = { HAWAT_ATTR_MID() => $group_id };
    $flt = {} if $group_id eq 'all';

    return $self->db->update(
                $flt,
                { '$set' => {'rep_compress' => 0 }},
                { 'multiple' => 1 }
            );
}

#-------------------------------------------------------------------------------

# build_filter($$)
#
#  Usage     : my $filter = $c->model('Groups')->build_filter($filter_object, $eventdb);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub build_filter($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter_object, $eventdb) = @_;
    die "Filter object must be given as argument\n" unless $filter_object and ref $filter_object eq 'HASH';

    my $e = [];
    if ($filter_object->{'type'} eq TYPE_SIMPLE) {
        $filter_object->{'analyzers'}  = [] if not $filter_object->{'analyzers'};
        $filter_object->{'categories'} = [] if not $filter_object->{'categories'};
        $filter_object->{'analyzers'}  = [$filter_object->{'analyzers'}]  if not ref $filter_object->{'analyzers'}  eq 'ARRAY';
        $filter_object->{'categories'} = [$filter_object->{'categories'}] if not ref $filter_object->{'categories'} eq 'ARRAY';
        if ($filter_object->{'ips'}) {
            unless (ref $filter_object->{'ips'} eq 'ARRAY') {
                chomp($filter_object->{'ips'});
                $filter_object->{'ips'} = [split(/[,;\s]+/, $filter_object->{'ips'})];
            }
        }
        else {
            $filter_object->{'ips'} = [];
        }

        # Perform the filter sanity validation
        #if (@{$filter_object->{'analyzers'}} and @{$filter_object->{'categories'}}) {
        #    foreach my $anl (@{$filter_object->{'analyzers'}}) {
        #        foreach my $cls (@{$filter_object->{'categories'}}) {
        #            push(@$e, "Invalid combination '$anl':'$cls' for analyzer:classification pair") unless exists $eventdb->{by_anl_cls}->{$anl}->{$cls};
        #        }
        #    }
        #}

        my @sections = ();
        if (@{$filter_object->{'analyzers'}}) {
            my %h = map { $_, 1 } @{$filter_object->{'analyzers'}};
            push(@sections, 'Node/Name IN ["'.join('","', (sort keys %h)).'"]');
        }
        if (@{$filter_object->{'categories'}}) {
            my %h = map { $_, 1 } @{$filter_object->{'categories'}};
            push(@sections, 'Category IN ["'.join('","', (sort keys %h)).'"]');
        }
        if (@{$filter_object->{'ips'}}) {
            my %h = map { $_, 1 } @{$filter_object->{'ips'}};
            push(@sections, 'Source/IP4 IN ["'.join('","', (sort keys %h)).'"]');
        }
        $filter_object->{'filter'} = join(' AND ', @sections);
    }
    else {
        $filter_object->{'analyzers'} = [];
        $filter_object->{'categories'} = [];
        $filter_object->{'ips'} = [];
    }
    return ($filter_object, $e);
}

#-------------------------------------------------------------------------------

# validate($)
#
#  Usage     : my $id = $c->model('Groups')->validate($group_object);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validate($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_object,) = @_;
    die "Group object must be given as argument\n" unless $group_object and ref $group_object eq 'HASH';

    my @errors = ();

    unless ($group_object->{HAWAT_ATTR_MID()}) {
        push(@errors, ['id','Unique group identifier is mandatory']);
    }
    unless ($group_object->{HAWAT_ATTR_SOURCE()}) {
        push(@errors, ['source','Entry source is mandatory']);
    }
    unless ($group_object->{'rep_mode'}) {
        push(@errors, ['rep_mode','Reporting mode is mandatory']);
    }
    unless ($group_object->{'rep_mode'} =~ /summary|extra|both/) {
        push(@errors, ['rep_mode',"Invalid reporting mode '$group_object->{rep_mode}'"]);
    }
    unless ($group_object->{'rep_template'}) {
        push(@errors, ['rep_template','Reporting template is mandatory']);
    }
    unless ($group_object->{'rep_template'} =~ /default|english/) {
        push(@errors, ['rep_template',"Invalid reporting template '$group_object->{rep_template}'"]);
    }
    unless ($group_object->{'rep_attach'}) {
        push(@errors, ['rep_attach','Reporting attachment is mandatory']);
    }
    unless ($group_object->{'rep_attach'} =~ /all|json|csv|none/) {
        push(@errors, ['rep_attach',"Invalid reporting attachment type '$group_object->{rep_attach}'"]);
    }
    # Checkbox provides value 'on', if checked, but we want integers
    if ($group_object->{'rep_redirect'}) {
        $group_object->{'rep_redirect'} = 1;
    }
    else {
        $group_object->{'rep_redirect'} = 0;
    }
    if ($group_object->{'rep_mute'}) {
        $group_object->{'rep_mute'} = 1;
    }
    else {
        $group_object->{'rep_mute'} = 0;
    }
    if ($group_object->{'rep_compress'}) {
        $group_object->{'rep_compress'} = 1;
    }
    else {
        $group_object->{'rep_compress'} = 0;
    }
    if ($group_object->{'rep_timing_type'} eq 'custom') {
        unless ($group_object->{'rep_timing'}) {
            push(@errors, ['rep_timing',"You have not specified custom report timing settings"]);
        }
        else {
            my ($n, $v);
            foreach my $s ('low','medium','high','critical') {
                unless ($group_object->{'rep_timing'}->{$s}){
                    push(@errors, ['rep_timing',"You have not specified timing settings for '$s' severity"]);
                    last;
                }
                foreach my $t ('period','threshold','relapse') {
                    $n = $group_object->{'rep_timing'}->{$s}->{$t};
                    unless (defined $n){
                        push(@errors, ['rep_timing',"You have not specified '$t' settings for '$s' severity"]);
                        last;
                    }
                    $v = $PERIODS_H->{$n};
                    unless (defined $v){
                        push(@errors, ['rep_timing',"Unknown value '$n' of '$t' setting for '$s' severity"]);
                        last;
                    }
                    if ($PERIODS_H->{$TIMING_THRS->{$s}->{$t}->{l}} > $v){
                        push(@errors, ['rep_timing',"You have specified too low value '$n' of '$t' setting for '$s' severity"]);
                        last;
                    }
                    if ($PERIODS_H->{$TIMING_THRS->{$s}->{$t}->{h}} < $v){
                        push(@errors, ['rep_timing',"You have specified too high value '$n' of '$t' setting for '$s' severity"]);
                        last;
                    }
                }
            }
        }
    }
    else {
        delete $group_object->{'rep_timing'} if exists $group_object->{'rep_timing'};
    }
    return @errors;
}

# validateFilter($$)
#
#  Usage     : my $id = $c->model('Groups')->validateFilter($filter_object, $eventdb);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validateFilter($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter_object, $eventdb) = @_;
    die "Filter object must be given as argument\n" unless $filter_object and ref $filter_object eq 'HASH';

    my @errors = ();
    my $e;
    unless ($filter_object->{HAWAT_ATTR_MID()}) {
        push(@errors, ['id','Unique group filter identifier is mandatory']);
    }
    unless ($filter_object->{HAWAT_ATTR_MID()} =~ /^[-_ a-zA-Z0-9]+$/) {
        push(@errors, ['id','Filter identifier must contain only following characters: [-_ a-zA-Z0-9]']);
    }
    unless ($filter_object->{HAWAT_ATTR_DESC()}) {
        push(@errors, ['description','Group filter description is mandatory']);
    }
    unless ($filter_object->{'type'} and ($filter_object->{'type'} eq TYPE_SIMPLE or $filter_object->{'type'} eq TYPE_ADVANCED)) {
        push(@errors, ['filter','Type of the reporting filter is mandatory, please select appropriate radio button']);
    }
    if ($filter_object->{validfrom} and not $filter_object->{validfrom} =~ /^\d+$/) {
        my $orig = $filter_object->{validfrom};
        eval {
            $filter_object->{validfrom} = Value::Convertor->datestr_to_unixs($filter_object->{validfrom});
        };
        push(@errors, ['validfrom',"Invalid timestamp value '$orig'"]) unless ($filter_object->{validfrom});
    }
    if ($filter_object->{validto} and not $filter_object->{validto} =~ /^\d+$/) {
        my $orig = $filter_object->{validto};
        eval {
            $filter_object->{validto} = Value::Convertor->datestr_to_unixs($filter_object->{validto});
        };
        push(@errors, ['validto',"Invalid timestamp value '$orig'"]) unless ($filter_object->{validto});
    }
    # Checkbox provides value 'on', if checked, but we want integers
    if ($filter_object->{enabled}) {
        $filter_object->{enabled} = 1;
    }
    else {
        $filter_object->{enabled} = 0;
    }
    ($filter_object, $e) = $self->build_filter($filter_object, $eventdb);
    if (@$e) {
        foreach (@$e) { push(@errors, ['categories', $_]); }
    }
    elsif (not $filter_object->{'filter'}) {
        push(@errors, ['filter','Reporting filter specification is mandatory']);
    }
    else {
        my ($f, $e) = $self->parseFilter($filter_object->{'filter'});
        push(@errors, ['filter','Invalid filter specification: '.$e]) if $e;

        #$filter_object->{'filter'} = $f->to_string();
    }

    return @errors;
}
# parseFilter($)
#
#  Usage     : my $id = $c->model('Groups')->parseFilter($filter_string);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub parseFilter($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter_string,) = @_;
    confess "Filter string must be given as argument\n" unless $filter_string;

    Log::Loger->notice(__PACKAGE__, "[Model::Groups] Parsing filter from string '$filter_string'");
    my $f;
    eval {
        $f = $self->ffactory->filter_from_string($filter_string);
    };
    Log::Loger->notice(__PACKAGE__, "[Model::Groups] Done parsing filter from string '$filter_string'");
    return (undef, $@) if ($@);
    return ($f, undef);

}

# validateNetwork($)
#
#  Usage     : my $id = $c->model('Whois')->validateNetwork($network_object);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validateNetwork($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($network_object,) = @_;
    die "Whois object must be given as argument\n" unless $network_object and ref $network_object eq 'HASH';

    my @errors = ();

    unless ($network_object->{HAWAT_ATTR_MID()}) {
        push(@errors, ['id','Network unique identifier is mandatory']);
    }
    unless ($network_object->{HAWAT_ATTR_MID()} =~ /^[-_ a-zA-Z0-9]+$/) {
        push(@errors, ['id','Network identifier must contain only following characters: [-_ a-zA-Z0-9]']);
    }
    unless ($network_object->{network}) {
        push(@errors, ['network','Network specification is mandatory']);
    }
    unless ($network_object->{netname}) {
        push(@errors, ['netname','Netname is mandatory']);
    }
    #if ($network_object->{resolved_abuses}) {
    #   unless (ref $network_object->{resolved_abuses} eq 'ARRAY') {
    #       $network_object->{resolved_abuses} = [split(/[,; ]/, $network_object->{resolved_abuses})];
    #   }
    #}

    my ($min, $max);
    my ($type, @matches) = Value::Convertor->detect_ip($network_object->{network});
    if ($type eq Value::Convertor::IPV4ADDR_STRING()) {
        $min = $max = Value::Convertor->ipv4str_to_int($network_object->{network});
        $network_object->{'type'}       = 'ipv4';
        $network_object->{'ipv4_start'} = Value::Convertor->int_to_ipv4str($min);
        $network_object->{'ipv4_end'}   = Value::Convertor->int_to_ipv4str($max);
    }
    elsif ($type eq Value::Convertor::IPV4CIDR_STRING()) {
        ($min, $max) = Value::Convertor->ipv4cidr_to_ints($network_object->{network});
        $network_object->{'type'}       = 'ipv4';
        $network_object->{'ipv4_start'} = Value::Convertor->int_to_ipv4str($min);
        $network_object->{'ipv4_end'}   = Value::Convertor->int_to_ipv4str($max);
    }
    elsif ($type eq Value::Convertor::IPV4RNG_STRING()) {
        ($min, $max) = Value::Convertor->ipv4rngstr_to_ints($network_object->{network});
        $network_object->{'type'}       = 'ipv4';
        $network_object->{'ipv4_start'} = Value::Convertor->int_to_ipv4str($min);
        $network_object->{'ipv4_end'}   = Value::Convertor->int_to_ipv4str($max);
    }
    else {
        push(@errors, ['network',"Invalid value '$network_object->{network}' for IP address, network CIDR or IP range"]);
    }
    if ($min and $max) {
        $network_object->{'first'} = $min;
        $network_object->{'last'}  = $max;
    }

    return @errors;
}

# validateNetworkBulk($)
#
#  Usage     : my $id = $c->model('Whois')->validateNetworkBulk($form);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub validateNetworkBulk($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($form,) = @_;
    die "Form data must be given as argument\n" unless $form and ref $form eq 'HASH';

    my @errors = ();

    unless ($form->{abuse_group}) {
        push(@errors, ['abuse_group','Abuse group is mandatory']);
    }
    unless ($form->{networks}) {
        push(@errors, ['network','Network specifications are mandatory']);
    }
    unless ($form->{netname}) {
        push(@errors, ['netname','Netname is mandatory']);
    }

    return @errors;
}

#-------------------------------------------------------------------------------

# removeFilters(;$)
#
#  Usage     : my $rv = $c->model('Groups')->removeFilters($group_id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub removeFilters(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($group_id, ) = @_;

    # Remove filter information from all groups
    if (not $group_id) {
        my $rv = $self->db->update(
                { },
                { '$set' => { 'rep_filters' => [] } },
                { 'multiple' => 1 }
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            return ("Successfully removed all filter information from all '$rv->{n}' groups", undef);
        }
        else {
            return (undef, "Unable to remove all filter information from all groups");
        }
    }
    # Remove filter information from particular group
    elsif ($group_id) {
        my $rv = $self->db->update(
                { HAWAT_ATTR_MID() => $group_id },
                { '$set' => { 'rep_filters' => [] } },
            );
        if ($rv->{ok} and $rv->{n} > 0) {
            return ("Successfully removed all filter information from '$group_id' group", undef);
        }
        else {
            return (undef, "Unable to remove all filter information from '$group_id' group");
        }
    }
}

# importFilters($)
#
#  Usage     : my $users = $c->model('Groups')->importFilters($);
#  Purpose   : Import events from external file
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub importFilters($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filename, $c) = @_;

    my $cfg_parser = Config::JSON->new();

    ## Load the templates from external file
    my ($config, $error);
    ($config, $error) = $cfg_parser->config($filename);
    die 'Filter file syntax error: ' . $error . "\n" unless $config;

    my $filters = $config->{filters};
    my $result = [];
    my $errors = [];
    my $filter_list;

    $c->log->info('IMPORT: File parsed');

    foreach my $group (sort keys %$filters) {
        next unless $group;
        $filter_list = $filters->{$group};

        foreach my $filter (@$filter_list) {
            $c->log->info('IMPORT: $group '.$filter->{id});

            my $rv = $self->addFilter($group, $filter);
            $c->log->info('IMPORT: $group '.$filter->{id}.' ADDED');
            if ($rv and $rv->{ok} and $rv->{n}) {
                push(@$result, "Imported filter '$filter->{id}' for group '$group'");
            }
            else {
                push(@$errors, "Unable to import filter '$filter->{id}' for group '$group'");
            }
        }
    }

    return ($result, $errors);
}

# exportFilters()
#
#  Usage     : my $users = $c->model('Groups')->exportFilters();
#  Purpose   : Export events to external file
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub exportFilters()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my ($filter_list, $count, $count_all, $duration, $query) = $self->listFilters();

    my $filters = {};
    foreach my $fg (@$filter_list) {
        push (@{$filters->{$fg->{i}}}, @{$fg->{f}});
    }
    foreach my $fg (keys %$filters) {
        foreach my $f (@{$filters->{$fg}}) {
            $f->{id} = $f->{_id};
            delete $f->{_id};
            delete $f->{hits};
        }
    }

    my $json = {
            "version"     => 1.0,
            "timestamp"   => time,
            "description" => 'Exported reporting filters',
            "filters"     => $filters,
        };

    return to_json($json, {utf8 => 1, pretty => 1});
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal helper objects for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));
    $self->ffactory(Data::Filter::Factory->new_from_file(
                    $self->filter_grammar));
    $self->aggregator(Mentat::Storage::Mongo::Aggregator->new(
        database    => $self->database_name,
        collection  => $self->collection_name,
        sdatabase   => $self->sdatabase_name,
        scollection => $self->scollection_name,
        cache       => $self->cache_dir,
        ttl         => $self->cache_ttl,
    ));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
    die "Unable to initialize filter factory attribute\n" unless $self->ffactory;
    die "Unable to initialize database aggregator attribute\n" unless $self->aggregator;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
