package Hawat::Model::Statistics;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use Data::Dumper;

use Log::Loger;
use Value::Convertor;
use Mentat::Storage::Mongo;
use Mentat::Stats::Mongo;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class       => 'Hawat::Model::Statistics',
    constructor => 'new',
);

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# get_stats_proc($)
#
#  Usage     : my ($count, $charts) = $c->model('Statistics')->get_stats_proc();
#  Purpose   : Get all available processing statistics
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub get_stats_proc($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($stats_dir,) = @_;

    my $base_path = '/data/statistician';
    my $error_file = '/static/images/chart-missing.png';

    my $i = 0;
    my @proc_charts = (
        {
            id => 'nodename',
            label => 'Overview by node name',
            charts => [
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Statistics by node name for last 6 hours',
                    src   => 'nodename.l6hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Statistics by node name for last 24 hours',
                    src   => 'nodename.l24hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Statistics by node name for last week',
                    src   => 'nodename.lweek.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Statistics by node name for last 4 weeks',
                    src   => 'nodename.l4weeks.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Statistics by node name for last 3 months',
                    src   => 'nodename.l3months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Statistics by node name for last 6 months',
                    src   => 'nodename.l6months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Statistics by node name for last 12 months',
                    src   => 'nodename.l12months.png',
                },
            ],
        },
        {
            id => 'tnodename',
            label => 'Totals by node name',
            charts => [
                #-------------------------------------------------------------------
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Total statistics by node name for last 6 hours',
                    src   => 'nodename.l6hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Total statistics by node name for last 24 hours',
                    src   => 'nodename.l24hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Total statistics by node name for last week',
                    src   => 'nodename.lweek-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Total statistics by node name for last 4 weeks',
                    src   => 'nodename.l4weeks-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Total statistics by node name for last 3 months',
                    src   => 'nodename.l3months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Total statistics by node name for last 6 months',
                    src   => 'nodename.l6months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Total statistics by node name for last 12 months',
                    src   => 'nodename.l12months-t.png',
                },
            ],
        },
        #*******************************************************************
        {
            id => 'nodesw',
            label => 'Overview by node SW',
            charts => [
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Statistics by node SW for last 6 hours',
                    src   => 'nodesw.l6hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Statistics by node SW for last 24 hours',
                    src   => 'nodesw.l24hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Statistics by node SW for last week',
                    src   => 'nodesw.lweek.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Statistics by node SW for last 4 weeks',
                    src   => 'nodesw.l4weeks.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Statistics by node SW for last 3 months',
                    src   => 'nodesw.l3months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Statistics by node SW for last 6 months',
                    src   => 'nodesw.l6months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Statistics by node SW for last 12 months',
                    src   => 'nodesw.l6months.png',
                },
            ],
        },
        {
            id => 'tnodesw',
            label => 'Totals by node SW',
            charts => [
                #-------------------------------------------------------------------
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Total statistics by node SW for last 6 hours',
                    src   => 'nodesw.l6hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Total statistics by node SW for last 24 hours',
                    src   => 'nodesw.l24hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Total statistics by node SW for last week',
                    src   => 'nodesw.lweek-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Total statistics by node SW for last 4 weeks',
                    src   => 'nodesw.l4weeks-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Total statistics by node SW for last 3 months',
                    src   => 'nodesw.l3months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Total statistics by node SW for last 6 months',
                    src   => 'nodesw.l6months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Total statistics by node SW for last 12 months',
                    src   => 'nodesw.l12months-t.png',
                },
            ],
        },
        #*******************************************************************
        {
            id => 'category',
            label => 'Overview by event category',
            charts => [
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Statistics by event category for last 6 hours',
                    src   => 'category.l6hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Statistics by event category for last 24 hours',
                    src   => 'category.l24hours.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Statistics by event category for last week',
                    src   => 'category.lweek.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Statistics by event category for last 4 weeks',
                    src   => 'category.l4weeks.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Statistics by event category for last 3 months',
                    src   => 'category.l3months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Statistics by event category for last 6 months',
                    src   => 'category.l6months.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Statistics by event category for last 12 months',
                    src   => 'category.l6months.png',
                },
            ],
        },
        {
            id => 'tcategory',
            label => 'Totals by event category',
            charts => [
                #-------------------------------------------------------------------
                {
                    id    => (++$i).'',
                    menu  => '6 hours',
                    title => 'Total statistics by event category for last 6 hours',
                    src   => 'category.l6hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '24 hours',
                    title => 'Total statistics by event category for last 24 hours',
                    src   => 'category.l24hours-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => 'Week',
                    title => 'Total statistics by event category for last week',
                    src   => 'category.lweek-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '4 weeks',
                    title => 'Total statistics by event category for last 4 weeks',
                    src   => 'category.l4weeks-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '3 months',
                    title => 'Total statistics by event category for last 3 months',
                    src   => 'category.l3months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '6 months',
                    title => 'Total statistics by event category for last 6 months',
                    src   => 'category.l6months-t.png',
                },
                {
                    id    => (++$i).'',
                    menu  => '12 months',
                    title => 'Total statistics by event category for last 12 months',
                    src   => 'category.l12months-t.png',
                },
            ],
        },
    );

    my $counter = 0;
    foreach my $group (@proc_charts) {
        foreach my $chart (@{$group->{charts}}) {
            unless (-f "$stats_dir/".$chart->{src}) {
                $chart->{src} = $error_file;
                $chart->{ok}  = 0;
            }
            else {
                $counter++;
                $chart->{src} = $base_path."/".$chart->{src};
                $chart->{ok}  = 1;
            }
        }
    }
    return ($counter, \@proc_charts);
}

# get_stats_db($;@)
#
#  Usage     : my $db_stats = $c->model('Statistics')->get_stats_db($database, @collections);
#  Purpose   : Get all available database statistics
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub get_stats_db($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($database_name, @collections) = @_;

    my $db_stats = Mentat::Stats::Mongo->new();
    return $db_stats->analyze($database_name, @collections);
}

# get_stats_col_alerts($;@)
#
#  Usage     : my $db_stats = $c->model('Statistics')->get_stats_col_alerts($database);
#  Purpose   : Get additional statistics for alerts collection
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub get_stats_col_alerts($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($database_name, @collections) = @_;

    my $db_stats = Mentat::Stats::Mongo->new();
    return $db_stats->analyze_alerts($database_name);
}

1;
